##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup

class TestReferenceIdCooking(PloneTestCase.PloneTestCase):
    '''Test the id cooking mechanism'''

    def afterSetUp(self):
        pass

    # some utility methods

    def getRawBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        bf = getattr(uf, 'bib_folder')
        return bf

    def getEmptyBibFolder(self,
                          idcooker_id='etal',
                          use_pids_on_import=True,
                          cook_ids_on_bibref_creation=False,
                          cook_ids_after_bibref_edit=False):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        bf = getattr(uf, 'bib_folder')
        bf.setReferenceIdCookingMethod(value=idcooker_id)
        bf.setUseParserIdsOnImport(value=use_pids_on_import)
        bf.setCookIdsOnBibRefCreation(value=cook_ids_on_bibref_creation)
        bf.setCookIdsAfterBibRefEdit(value=cook_ids_after_bibref_edit)
        return bf

    def getPopulatedBibFolder(self,
                              idcooker_id='etal',
                              use_pids_on_import=True,
                              cook_ids_on_bibref_creation=False,
                              cook_ids_after_bibref_edit=False):
        bf = self.getEmptyBibFolder()
        bf.setReferenceIdCookingMethod(value=idcooker_id)
        bf.setUseParserIdsOnImport(value=use_pids_on_import)
        bf.setCookIdsOnBibRefCreation(value=cook_ids_on_bibref_creation)
        bf.setCookIdsAfterBibRefEdit(value=cook_ids_after_bibref_edit)
        idcooking_source = open(setup.IDCOOKING_TEST_BIB, 'r').read()
        bf.processImport(idcooking_source, 'idcooking_test.bib')
        return bf   

    def processBibRefObject(self, bibfolder=None, bibref_item=None, template_no=None, bibref_id=None, **kwargs):
    
        content_templates = (
                { 'type_name': 'BookReference',
                  'id': 'bookreference.2006-08-28.8979763101',
                  'authors': ({'lastname': 'Mueller', 'firstname': 'Felix', 'middlename': ''},
                              {'lastname': 'Leupelt', 'firstname': 'Maren', 'middlename': ''}, ),
                  'title': 'Eco Targets, Goal Functions, and Orientors',
                  'publication_year': '1997',
                  'publisher': 'Springer-Verlag',
                  'address': 'Berlin, Heidelberg, New York',
                },
                { 'type_name': 'ArticleReference',
                  'id': 'articlereference.2006-08-27.8979763101',
                  'authors': ({'lastname': 'Barkmann', 'firstname': 'Jan', 'middlename': ''},
                              {'lastname': 'Baumann', 'firstname': 'R.', 'middlename': ''},
                              {'lastname': 'Meyer', 'firstname': 'U.', 'middlename': ''},
                              {'lastname': 'Mueller', 'firstname': 'Felix', 'middlename': ''},
                              {'lastname': 'Windhorst', 'firstname': 'Wilhelm', 'middlename': ''}, ),
                  'title': 'On the role of Ecosystem Self-Organisation in Landscape Management - A Response',
                  'publication_year': '2001',
                  'journal': 'GAIA',
                  'volume': '10',
                  'number': '4',
                  'pages': '247-248',
                },
                { 'type_name': 'ArticleReference',
                  'id': 'articlereference.2006-08-26.8979763101',
                  'authors': ({'lastname': 'Dolnik', 'firstname': 'Christian', 'middlename': ''}, ),
                  'title': 'Agonimia allobata und Nachweise anderer seltener Flechten aus Schleswig-Holstein',
                  'publication_year': '2005',
                  'journal': 'Kieler Notizen Pflanzenkd. Schleswig-Holstein Hamb.',
                  'volume': '33',
                  'pages': '90-97',
                },
                { 'type_name': 'BookletReference',
                  'id': 'bookletreference.2006-08-25.8979763101',
                  'title': 'Kleines Plone Handbuch',
                  'publication_year': '2005',
                },
        )
        if bibfolder is not None:

            values = {}
            if template_no is not None:
                values = content_templates[template_no]

            if bibref_id is None and values.has_key('id'):
                bibref_id = values['id']

            # new bibref item in bibfolder
            if bibref_item is None:
                bibfolder.invokeFactory(type_name=values['type_name'], id=bibref_id )
                bibref_item = getattr(bibfolder, bibref_id)

            if values.has_key('id'):
               del values['id']
            if values.has_key('type_name'):
                del values['type_name']

            # replace **kwargs in values
            for key in kwargs.keys():

                # the authors field behaves weird, comes in as tuple-in-tuple
                if key in [ field.getName() for field in bibref_item.Schema().fields() ] and (key == 'authors'):
                    values[key] = kwargs[key][0]
                elif key in [ field.getName() for field in bibref_item.Schema().fields() ]:
                    values[key] = kwargs[key]

            bibref_item.processForm(values=values)

    # the individual tests

    def test_BibliographyToolIdCookingDefaults(self):
        bib_tool = self.portal.portal_bibliography
        default_idcooker_id = bib_tool.getDefaultIdCooker().getId()
        use_pids_on_import = bib_tool.useParserIdsOnImport()
        cook_ids_on_bibref_creation = bib_tool.cookIdsOnBibRefCreation()
        cook_ids_after_bibref_edit = bib_tool.cookIdsAfterBibRefEdit()

        self.failUnless(default_idcooker_id == 'etal')
        self.failUnless(use_pids_on_import == True)
        self.failUnless(cook_ids_on_bibref_creation == False)
        self.failUnless(cook_ids_after_bibref_edit == False)

    def test_BibliographyToolIdCookingSetup(self):

        self.setRoles(('Manager',))
        bib_tool = self.portal.portal_bibliography
        request = { 'default_idcooker_id': 'abbrev', 'use_pids_on_import': False, 'cook_ids_on_bibref_creation': True, 'cook_ids_after_bibref_edit': True, }
        for key in request.keys():
            self.app.REQUEST.set(key, request[key])
        bib_tool.prefs_bibliography_idcooking()

        default_idcooker_id = bib_tool.getDefaultIdCooker().getId()
        use_pids_on_import = bib_tool.useParserIdsOnImport()
        cook_ids_on_bibref_creation = bib_tool.cookIdsOnBibRefCreation()
        cook_ids_after_bibref_edit = bib_tool.cookIdsAfterBibRefEdit()

        self.failUnless(default_idcooker_id == 'abbrev')
        self.failUnless(use_pids_on_import == False)
        self.failUnless(cook_ids_on_bibref_creation == True)
        self.failUnless(cook_ids_after_bibref_edit == True)

    def test_BibliographyFolderIdCookingWithBibToolDefaults(self):

        bib_tool = self.portal.portal_bibliography
        default_idcooker_id = bib_tool.getDefaultIdCooker().getId()
        use_pids_on_import = bib_tool.useParserIdsOnImport()
        cook_ids_on_bibref_creation = bib_tool.cookIdsOnBibRefCreation()
        cook_ids_after_bibref_edit = bib_tool.cookIdsAfterBibRefEdit()

        bf = self.getRawBibFolder()
    
        self.failUnless(default_idcooker_id == bf.getReferenceIdCookingMethod())
        self.failUnless(use_pids_on_import == bf.getUseParserIdsOnImport())
        self.failUnless(cook_ids_on_bibref_creation == bf.getCookIdsOnBibRefCreation())
        self.failUnless(cook_ids_after_bibref_edit == bf.getCookIdsAfterBibRefEdit())

    def test_BibliographyFolderIdCookingWithBibToolCustomized(self):

        self.setRoles(('Manager',))
        bib_tool = self.portal.portal_bibliography
        request = { 'default_idcooker_id': 'abbrev', 'use_pids_on_import': False, 'cook_ids_on_bibref_creation': True, 'cook_ids_after_bibref_edit': True, }
        for key in request.keys():
            self.app.REQUEST.set(key, request[key])
        bib_tool.prefs_bibliography_idcooking()

        default_idcooker_id = bib_tool.getDefaultIdCooker().getId()
        use_pids_on_import = bib_tool.useParserIdsOnImport()
        cook_ids_on_bibref_creation = bib_tool.cookIdsOnBibRefCreation()
        cook_ids_after_bibref_edit = bib_tool.cookIdsAfterBibRefEdit()

        bf = self.getRawBibFolder()
    
        self.failUnless('abbrev' == bf.getReferenceIdCookingMethod())
        self.failUnless(False == bf.getUseParserIdsOnImport())
        self.failUnless(True == bf.getCookIdsOnBibRefCreation())
        self.failUnless(True == bf.getCookIdsAfterBibRefEdit())

    def test_ReferenceIdCookingOnBibRefCreation(self):

        bf = self.getEmptyBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=True)

        self.processBibRefObject(bibfolder=bf, template_no=0)
        self.processBibRefObject(bibfolder=bf, template_no=1)
        self.processBibRefObject(bibfolder=bf, template_no=2)

        self.failUnless( 'ML1997' in bf.contentIds() )
        self.failUnless( 'BBMMW2001' in bf.contentIds() )
        self.failUnless( 'Dol2005' in bf.contentIds() )

    def test_NoReferenceIdCookingOnBibRefCreation(self):

        bf = self.getEmptyBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=False)

        self.processBibRefObject(bibfolder=bf, template_no=0)
        self.processBibRefObject(bibfolder=bf, template_no=1)
        self.processBibRefObject(bibfolder=bf, template_no=2)

        self.failUnless( 'bookreference.2006-08-28.8979763101' in bf.contentIds() )
        self.failUnless( 'articlereference.2006-08-27.8979763101' in bf.contentIds() )
        self.failUnless( 'articlereference.2006-08-26.8979763101' in bf.contentIds() )

    def test_MyIdOnBibRefCreation(self):

        bf = self.getEmptyBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=False)

        self.processBibRefObject(bibfolder=bf, template_no=0, bibref_id='myid0', )
        self.processBibRefObject(bibfolder=bf, template_no=1, bibref_id='myid1', )
        self.processBibRefObject(bibfolder=bf, template_no=2, bibref_id='myid2', )

        self.failUnless( 'myid0' in bf.contentIds() )
        self.failUnless( 'myid1' in bf.contentIds() )
        self.failUnless( 'myid2' in bf.contentIds() )

    def test_ReferenceIdCookingAfterBibRefEdit(self):

        bf = self.getEmptyBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=True, cook_ids_after_bibref_edit=True)

        self.processBibRefObject(bibfolder=bf, template_no=0)
        self.failUnless( 'ML1997' in bf.contentIds() )

        ml1997 = getattr(bf, 'ML1997')
        edit_authors = ({'lastname': 'Fueller', 'firstname': 'Melix', 'middlename': ''}, {'lastname': 'Meupelt', 'firstname': 'Laren', 'middlename': ''}, ),
        self.processBibRefObject(bibfolder=bf, bibref_item=ml1997, authors=edit_authors, )

        self.failUnless( 'FM1997' in bf.contentIds() )

    def test_NoReferenceIdCookingAfterBibRefEdit(self):

        bf = self.getEmptyBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=True, cook_ids_after_bibref_edit=False)

        self.processBibRefObject(bibfolder=bf, template_no=0)
        self.failUnless( 'ML1997' in bf.contentIds() )

        ml1997 = getattr(bf, 'ML1997')
        edit_authors = ({'lastname': 'Fueller', 'firstname': 'Melix', 'middlename': ''}, {'lastname': 'Meupelt', 'firstname': 'Laren', 'middlename': ''}, ),
        self.processBibRefObject(bibfolder=bf, bibref_item=ml1997, authors=edit_authors, )

        self.failUnless( 'ML1997' in bf.contentIds() )

    def test_NoAuthorReferenceIdCookerBehaviour(self):

        bf = self.getPopulatedBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=True)

        self.processBibRefObject(bibfolder=bf, template_no=3)
        self.failUnless( 'nobody2005' in bf.contentIds() )

    def test_NextIdReferenceIdCookerBehaviour(self):

        bf = self.getEmptyBibFolder(idcooker_id='abbrev', cook_ids_on_bibref_creation=True, cook_ids_after_bibref_edit=True)

        self.processBibRefObject(bibfolder=bf, template_no=0,)
        item0 = getattr(bf, 'ML1997')
        self.processBibRefObject(bibfolder=bf, template_no=1,)
        item1 = getattr(bf, 'BBMMW2001')
        self.processBibRefObject(bibfolder=bf, template_no=2,)
        item2 = getattr(bf, 'Dol2005')
        self.processBibRefObject(bibfolder=bf, template_no=3,)
        item3 = getattr(bf, 'nobody2005')
        edit_authors = ({'lastname': 'Gabriel', 'firstname': 'Mike', 'middlename': ''}, ),
        self.processBibRefObject(bibfolder=bf, bibref_item=item0, authors=edit_authors, publication_year='2006' )
        self.processBibRefObject(bibfolder=bf, bibref_item=item1, authors=edit_authors, publication_year='2006' )
        self.processBibRefObject(bibfolder=bf, bibref_item=item2, authors=edit_authors, publication_year='2006' )

        self.failUnless( 'Gab2006' in bf.contentIds() )
        self.failUnless( 'Gab2006a' in bf.contentIds() )
        self.failUnless( 'Gab2006b' in bf.contentIds() )
        self.failUnless( 'nobody2005' in bf.contentIds() )

        # remove item 1
        bf.manage_delObjects(['Gab2006a',])

        # editing item 0 should leave id where it is (as opposed to appending it to the chain of nextIDs)
        self.processBibRefObject(bibfolder=bf, bibref_item=item0, title='REMOVED' )
        self.failUnless( item0.getId() == 'Gab2006' )

        # same authors for item 3 should result in Gab2006a (as item 1 has been removed from bibfolder)
        self.processBibRefObject(bibfolder=bf, bibref_item=item3, template_no=3, authors=edit_authors, publication_year='2006' )
        self.failUnless( item3.getId() == 'Gab2006a' )

        # remove item 3, edit item 2: Gab2006b -> Gab2006a
        bf.manage_delObjects(['Gab2006a',])
        self.processBibRefObject(bibfolder=bf, bibref_item=item2, template_no=2, authors=edit_authors, publication_year='2006' )
        self.failUnless( item2.getId() == 'Gab2006a' )


    def test_ReferenceIdCookerUsePIDsOnImport(self):

        bf = self.getPopulatedBibFolder(use_pids_on_import=True)

        self.failUnless( 'GrootBruinsBreeuwer2003' in bf.contentIds() )
        self.failUnless( 'AlibardiThompson2003' in bf.contentIds() )
        self.failUnless( 'CokeHunterIsazaKochGoatleyCarpenter2003' in bf.contentIds() )
        self.failUnless( 'TrapeMane2002' in bf.contentIds() )
        self.failUnless( 'Dolnik2005' in bf.contentIds() )

    def test_EtAlReferenceIdCooker(self):

        bf = self.getPopulatedBibFolder(idcooker_id='etal', use_pids_on_import=False)

        self.failUnless( 'GrootEtAl2003' in bf.contentIds() )
        self.failUnless( 'AlibardiThompson2003' in bf.contentIds() )
        self.failUnless( 'CokeEtAl2003' in bf.contentIds() )
        self.failUnless( 'TrapeMane2002' in bf.contentIds() )
        self.failUnless( 'Dolnik2005' in bf.contentIds() )

    def test_AbbrevReferenceIdCooker(self):

        bf = self.getPopulatedBibFolder(idcooker_id='abbrev', use_pids_on_import=False)

        self.failUnless( 'GBB2003' in bf.contentIds() )
        self.failUnless( 'AT2003' in bf.contentIds() )
        self.failUnless( 'CHIKGC2003' in bf.contentIds() )
        self.failUnless( 'TM2002' in bf.contentIds() )
        self.failUnless( 'Dol2005' in bf.contentIds() )

    def test_PloneReferenceIdCooker(self):

        bf = self.getPopulatedBibFolder(idcooker_id='plone', use_pids_on_import=False)

        self.failUnless( 'agonimia-allobata-und-nachweise-anderer-seltener-flechten-aus-schleswig-holstein' in bf.contentIds() )
        self.failUnless( 'molecular-genetic-evidence-for-parthenogenesis-in-the-burmese-python-python-molurus-bivittatus' in bf.contentIds() )
        self.failUnless( 'epidermal-differentiation-during-ontogeny-and-after-hatching-in-the-snake-liasis-fuscus-pythonidae-serpentes-reptilia-with-emphasis-on-the-formation-of-the-shedding-complex' in bf.contentIds() )
        self.failUnless( 'pharmacokinetics-and-tissue-concentrations-of-azithromycin-in-ball-pythons-python-regius' in bf.contentIds() )
        self.failUnless( 'the-snakes-of-senegal-an-annotated-species-list' in bf.contentIds() )

    def test_UmlautNormalization(self):

        bf = self.getPopulatedBibFolder(idcooker_id='etal', use_pids_on_import=False)

        self.failUnless( 'OOAAUUooaauu2005' in bf.contentIds() )

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestReferenceIdCooking))
    return suite

if __name__ == '__main__':
    framework()



from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import BaseSchema, Schema
    from Products.LinguaPlone.public import StringField, TextField, LinesField, ReferenceField, FileField
    from Products.LinguaPlone.public import SelectionWidget, ReferenceWidget, TextAreaWidget, FileWidget
    from Products.LinguaPlone.public import RichWidget, IdWidget, StringWidget
    from Products.LinguaPlone.public import DisplayList
else:
    from Products.Archetypes.public import BaseSchema, Schema
    from Products.Archetypes.public import StringField, TextField, LinesField, ReferenceField, FileField
    from Products.Archetypes.public import SelectionWidget, ReferenceWidget, TextAreaWidget, FileWidget
    from Products.Archetypes.public import RichWidget, IdWidget, StringWidget
    from Products.Archetypes.public import DisplayList

from Products.Archetypes.Widget import TypesWidget
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget \
     import ReferenceBrowserWidget

from Products.CMFBibliographyAT.marshall import BibtexMarshaller
from Products.ATExtensions.ateapi import FormattableNamesField, FormattableNamesWidget, \
                                         CommentField, CommentWidget


HeaderSchema = BaseSchema.copy()
HeaderSchema['id'].widget.macro_edit = "widgets/string"
# XXX ugly hack as long as IdWidget subclasses TypesWidget
HeaderSchema['id'].widget.size = 40
HeaderSchema['id'].widget.maxlength = 255
HeaderSchema['id'].widget.condition = 'python: not object.getBibFolder().getCookIdsAfterBibRefEdit()'

CookIdWarningField = StringField('shortname_cookid_warning',
                 schemata="default",
		 mode="rw",
		 accessor="getCookIdWarning",
		 widget = StringWidget(
		    label="Short Name",
		    label_msgid="label_shortname_cookid_warning",
		    description="There is ID re-cooking (after every edit / paste action) enabled on this item's parent bibliography folder. You cannot manually edit this item's short name. Modifications in this field will be ignored.",
		    description_msgid="help_shortname_cookid_warning",
		    i18n_domain="cmfbibliographyat",
		    modes=('view', 'edit'),
		    condition = 'python: object.getBibFolder().getCookIdsAfterBibRefEdit() and object.portal_membership.getAuthenticatedMember().getProperty("visible_ids", object.portal_memberdata.getProperty("visible_ids"))',
		 ),
)

HeaderSchema.addField(CookIdWarningField)

tmp_title = HeaderSchema['title']
tmp_title.is_duplicates_criterion=True
del HeaderSchema['title']

AuthorSchema = Schema((
    ReferenceField('member_publication_authors',
        relationship='authorOf',
        mutator='setMemberPublicationAuthors',
        languageIndependent=1,
        multiValued=1,
        widget=ReferenceWidget(label='Authors',
            description='',
            visible={'view':'invisible', 'edit':'invisible'},
            condition='python:object.showMemberAuthors()',
            i18n_domain="cmfbibliographyat",
            ),
        ),
    FormattableNamesField('authors', # was 'publication_authors',
        searchable = 1,
        required = 0,
        minimalSize = 2,
        subfields=('reference','firstnames','lastname', 'homepage'),
        subfield_sizes={'firstnames':20, 'lastname':20, 'homepage':15},
        subfield_vocabularies={'reference':'getSiteMembers'},
        subfield_conditions={'reference':'python:object.showMemberAuthors()'},
        subfield_maxlength={'homepage': 250,},
	is_duplicates_criterion=True,
        widget=FormattableNamesWidget(label="Authors",
            label_msgid="label_authors",
            description="If possible, always fill in the complete authors' / editors' names.",
            description_msgid="help_authors",
            i18n_domain="cmfbibliographyat",
        ) ,
    ),
))

CoreSchema = Schema((
    tmp_title,
    StringField('publication_year',
        searchable=1,
        required=1,
        languageIndependent=1,
	is_duplicates_criterion=True,
        widget=StringWidget(
            label="Publication Year",
            label_msgid="label_publication_year",
            description_msgid="help_publication_year",
            description="The year of publication. For unpublished works, please specify the year of composure. This field should contain a year number like '1984', but it can also process notes like 'in print' etc.",
            i18n_domain="cmfbibliographyat",
            ),
        ),
    TextField('abstract',
        searchable=1,
        required=0,
        default_content_type='text/html',
        default_output_type='text/x-html-safe',
        allowable_content_types=('text/structured',
                                 'text/restructured',
                                 'text/html',
                                 'text/plain',),
        accessor="getAbstract",
        edit_accessor="editAbstract",
        mutator="setAbstract",
        widget=RichWidget(
            label="Abstract",
            label_msgid="label_abstract",
            description="An abstract of the referenced publication. Please contact your portal's reviewers if unsure about the site's default language for abstracts in bibliographical references.",
            description_msgid="help_abstract",
            i18n_domain="cmfbibliographyat",
            rows=5,
            ),
        ),
    # full text and printable versions
    CommentField('explain_links',
                 schemata="full text",
                 comment = """\
                 There are several ways to make reference to
                 the original paper:

                   - A link to an online version

                   - A link to a printable (pdf) version

                   - Upload a printable (pdf) file""",
                 comment_msgid="comment_explain_links",
                 comment_type = "text/structured",
		 widget = CommentWidget(
                    i18n_domain="cmfbibliographyat",
                 ),
    ), 
    StringField('publication_url',
        schemata="full text",
        required=0,
        searchable=0,
        ## validators=('isURL',),    # re-enable if it doesn't fail
                                     # for empty values any more
        widget=StringWidget(
            label="Online URL",
            label_msgid="label_url",
            description="The (external) URL to get to an online version of the referenced resource.",
            description_msgid="help_url",
            i18n_domain="cmfbibliographyat",
            ),
        ),
    StringField('pdf_url',
        schemata="full text",
        required=0,
        searchable=0,
        ## validators=('isURL',),    # re-enable if it doesn't fail
                                     # for empty values any more
        widget=StringWidget(
            label="PDF URL",
            label_msgid="label_pdfref_url",
            description="The (external) URL to retrieve a printable version (PDF file) of the referenced resource from.",
            description_msgid="help_pdfref_url",
            i18n_domain="cmfbibliographyat",
            ),
        ),
    ReferenceField('pdf_file',
                    schemata="full text",
                    relationship="printable_version_of",
                    multiValued=0,
                    required=0,
                    allowed_types=("PDF File",),
                    # allowed_types=("PDF File", "File"),
                    widget=ReferenceBrowserWidget(
                        label="AT Reference to Printable File (PDF Format)",
                        label_msgid="label_pdf_file",
                        description="This is AT field is hidden to anyone but portal managers. It refers to the associated PDF document on this site (if any). Use it for repair if the PDF file association of this bibliographical entry is broken.",
                        description_msgid="help_pdf_file",
                        addable=0,
                        force_close_on_insert=True, 
                        destination='getPdfFolderPath',
                        startup_directory='getPdfFolderPath',
                        i18n_domain = "cmfbibliographyat",
                        visible={'edit': 'visible', 'view': 'invisible',},
                        condition="python: object.portal_membership.checkPermission('ManagePortal', object)",
                    ),
    ),
    FileField('uploaded_pdfFile',
                    schemata="full text",
                    languageIndependent=True,
                    default_content_type = "application/pdf",
                    mutator='setUploaded_pdfFile',
                    edit_accessor='editUploaded_pdfFile',
                    accessor='getUploaded_pdfFile', 
                    #validators = (('isNonEmptyFile', V_REQUIRED),),
                    widget = FileWidget(
                                label= "Printable PDF File",
                                label_msgid = "label_upload_pdffile_from_bibrefitem",
                                description = "If not in conflict with any copyright issues, use this field to upload a printable version (PDF file) of the referenced resource.",
                                description_msgid = "help_upload_pdffile_from_bibrefitem",
                                i18n_domain = "cmfbibliographyat",
                                show_content_type = True,
                                condition="python:object.portal_bibliography.allowPdfUploadPortalPolicy() and object.isPdfUploadAllowedForThisType()",
                    ),
    ),
    ReferenceField('is_duplicate_of',
                    relationship="hasDuplicates",
                    multiValued=1,
                    required=0,
                    allowed_types='getBibReferenceTypes',
                    widget=ReferenceWidget(
                        visible={'edit': 'invisible', 'view': 'invisible',},
                    ),
    ),
))

TrailingSchema = Schema((
    StringField('publication_month',
        searchable=1,
        required=0,
        languageIndependent=1,
	is_duplicates_criterion=True,
        widget=StringWidget(
            label="Publication Month",
            label_msgid="label_publication_month",
            description_msgid="help_publication_month",
            description="Month of publication (or writing, if not published).",
            i18n_domain="cmfbibliographyat",
            ),
        ),
    TextField('note',
        searchable=1,
        required=0,
	is_duplicates_criterion=True,
        widget=TextAreaWidget(label="Note",
            label_msgid="label_note",
            description_msgid="help_note",
            description="Any additional information that can help the reader. The first word should be capitalized.",
            i18n_domain="cmfbibliographyat",
            ),
        ),
    TextField('annote',
        searchable=1,
        required=0,
	is_duplicates_criterion=True,
        widget=TextAreaWidget(label="Annote",
            label_msgid="label_annote",
            description_msgid="help_annote",
            description="Any annotation that you do not wish to appear in rendered (bibtex) bibliographies.",
            i18n_domain="cmfbibliographyat",
            ),
        ),
    ), marshall = BibtexMarshaller())




############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""BibliographyParser main class"""

# Python stuff
import re

# Zope stuff
from Interface import Interface
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from Acquisition import Implicit
import Products


class IBibliographyParser(Interface):
    """
    Interface for the input parsers of the bibliography tool.
    """
    def isAvailable():
        """
        are all pre-requisites for this parser fullfilled?
        """
          
    def isEnabled():
        """
        is this parser enabled by the portal manager (default: yes)?
        """
          
    def getFormatName():
        """
        returns the name of the format
        """

    def getFormatExtension():
        """
        returns the filename extension of the format
        """
        
    def getDelimiter():
        """
        returns the delimiter used to split a list of entries into pieces
        """

    def getPattern():
        """
        returns the pattern used to parse a single entry
        """

    def setDelimiter(delimiter="\n\n", flags=None):
        """
        sets the delimiter used to split a list of entries into pieces
        the 'delimiter' argument and the flags are passed to 're.compile'
        """

    def setPattern(pattern="\n", flags=None):
        """
        sets the pattern used to parse a single entry
        the 'pattern' argument is passed to 're.compile')
        """

    def checkFormat(source):
        """
        checks whether source has the right format
        returns true (1) if so and false (0) otherwise
        """
        
    def getEntries(source):
        """
        splits a (text) file with several entries
        parses the entries
        returns a list of the parsed entries
        """

    def splitSource(source):
        """
        splits a (text) file with several entries
        returns a list of those entries
        """

    def parseEntry(entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """


class BibliographyParser(SimpleItem, PropertyManager):
    """
    Base class for the input parser of the bibliography tool.
    """
    __implements__ = (IBibliographyParser,)

    meta_type = 'Bibliography Parser'
    parser_enabled = True
    format = {'name':'name of the format',
              'extension':'typical filename extension'
              }

    manage_options = (
        PropertyManager.manage_options + 
        SimpleItem.manage_options
    ) 
    _properties = PropertyManager._properties + (
        {'id':'parser_enabled',
         'type':'boolean',
         'mode':'w',
         },
    )
    
    security = ClassSecurityInfo()

    delimiter = "\n\n"       # used to split a text into a list of entries
    pattern = r'(^.{0,4}- )' # the Medline pattern as default

    def __init__(self, id, title=''):
        """
        minimal initialization
        """
        self.id = id
        self.title = title
        self.format = format

    def isAvailable(self):
        """ by default parser is available, override in specific parser's code
            if there are some hurdles to take before parsing is possible 
        """
        return True

    def isEnabled(self):
        """ if parser is enabled or not can be configured in the PropertyManager
        """
        return self.parser_enabled

    def Description(self):
        """
        a short text that explains the target format of the renderer
        """
        domain='cmfbibliographyat'
        msgid='help_parser_%s' % self.getId()
        return self.translate(domain=domain, msgid=msgid, default='%s' % self.__doc__)
                                                        
    def getFormatName(self):
        """ returns the name of the format """
        return self.format.get('name', 'No name specified')

    def getFormatExtension(self):
        """ returns the filename extension of the format """
        return self.format.get('extension', 'no extension specified')

    def getDelimiter(self):
        """
        returns the delimiter used to split a list of entries into pieces
        """
        return self.delimiter

    def getPattern(self):
        """
        returns the pattern used to parse a single entry
        """
        return self.pattern

    def setDelimiter(self, delimiter="\n\n", flags=None):
        """
        sets the delimiter used to split a list of entries into pieces
        the 'delimiter' argument and the flags are passed to 're.compile'
        """
        if flags:
            self.delimiter = re.compile(delimiter, flags)
        else:
            self.delimiter = re.compile(delimiter)
        return None

    def setPattern(self, pattern="\n", flags=None):
        """
        sets the pattern used to parse a single entry
        the 'pattern' argument is passed to 're.compile')
        """
        if flags:
            self.pattern = re.compile(pattern, flags)
        else:
            self.pattern = re.compile(pattern)
        return None

    def checkFormat(source):
        """
        checks whether source has the right format
        returns true (1) if so and false (0) otherwise
        """
        pass # needs to be provided by the individual parsers
    
    def splitSourceFile(self, source):
        """
        splits a (text) file with several entries
        returns a list of those entries
        """
        if hasattr(self, 'preprocess'):
            source = self.preprocess(source)
        return self.delimiter.split(source)

    def parseEntry(self, entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        pass  # needs to be overwriten by the individual parser
        
    def getEntries(self, source):
        """
        splits a (text) file with several entries
        parses the entries
        returns a list of the parsed entries
        """
        return [self.parseEntry(entry) \
                for entry in self.splitSourceFile(source)]

                
InitializeClass(BibliographyParser)

class ParserFolder(Folder):
    """
    A folder that only offers to add objects that implement the
    IBibliographyParser interface.
    """
    meta_type = 'Parser Folder'

    id = 'Parsers'
    title = "BibliographyTool's parser folder"

    # we don't want 'View'
    manage_options = ( Folder.manage_options[0], ) \
                     + Folder.manage_options[2:]
    index_html = None

    def __init__(self, id, title=''):
        """
        minimal initialization
        """
        self.id = id
        # self.title = title
    
    def all_meta_types(self):
        product_infos = Products.meta_types
        possibles = []
        for p in product_infos:
            try:
                if IBibliographyParser in p.get('interfaces', []):
                    possibles.append(p)
            except TypeError:
                pass
        definites = map(lambda x: x.meta_type, self.objectValues())
        return filter(lambda x,y=definites: x['name'] not in y, possibles)

InitializeClass(ParserFolder)


class EntryParseError(Implicit):
    """Parsers can return instances of this class when the parsing
    of an entry fails for whatever reason.
    The description is then taken and added to the import_report.
    """

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    def __init__(self, description):
        self.description = description.replace('\n', '\\n')

    security.declarePublic('get')
    def get(self, attr_name):
        # XXX Hack! import.cpy does::
        #    if entry.get('title'):
        #        ...
        # So we need to mimic that here
        if attr_name == 'description':
            return self.description
        else:
            return 'EntryParseError'

InitializeClass(EntryParseError)



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Unpublished reference class"""

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry, BaseEntrySchema

UnpublishedSchema = BaseEntrySchema.copy()
UnpublishedSchema.get('authors').required = 1
# YES!!! note is a required field for UnpublishedReferences!!!
UnpublishedSchema.get('note').required = 1
# normally the publication_year for UnpublishedReferences is optional, but
# in CMFBAT we better force the user to enter something here (better not 
# irritate portal_catalog...).
UnpublishedSchema.get('publication_year').required = 1 

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
UnpublishedSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }


class UnpublishedReference(BaseEntry):
    """Content type to make reference to a unpublished document.
    """
    archetype_name = "Unpublished Reference"

registerType(UnpublishedReference, PROJECTNAME)



############################################################################
#                                                                          #
#             copyright (c) 2004 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""XMLRenderer (MODS) class"""

# Python stuff
import os, types

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# CMF stuff
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.renderers.base \
     import IBibliographyRenderer, BibliographyRenderer

from Products.CMFBibliographyAT.utils import _default_encoding

class XMLRenderer(BibliographyRenderer):
    """
    A specific XML (MODS) intermediate renderer. The XML intermediate format is conform to  
    the Library of Congress's Metadata Object Description Schema (MODS). This is a very flexible standard 
    that should prove quite useful as the number of tools that directly interact with it increase. 
    """
    # depends on the BibTeX renderer

    __implements__ = (IBibliographyRenderer ,)

    meta_type = "XML Renderer"

    format = {'name':'XML (MODS)',
              'extension':'xml'}

    def __init__(self,
                 id = 'xml_mods',
                 title = "XML (MODS) - transforms " \
                 "bibtex renderer output"):
        """
        initializes id and title
        """
        self.id = id
        self.title = title

    def isAvailable(self):
        """ test if transforming from BibTex to XML is possible...
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return bib_tool.isTransformable('bib', 'xml')
                                                    
    def render(self, objects, output_encoding=None, msdos_eol_style=False, **kwargs):
        """
        renders a bibliography object in XML (MODS) format
        """
        if type(objects) not in (types.ListType, types.TupleType):
            objects = [objects]   
        bib_tool = getToolByName(objects[0], 'portal_bibliography')
        bibtex_source = bib_tool.render(objects, 'bib', output_encoding=_default_encoding, msdos_eol_style=msdos_eol_style)

        # open a pipe
        (fi, fo, fe) = os.popen3('bib2xml ', 't')
        # provide the input
        fi.write(bibtex_source)
        fi.close()
        # get the output
        xml = fo.read()
        fo.close()
        # get the staus/error message
        # (this isn't used but we don't want it in the output)
        error = fe.read()
        fe.close()
        # done
        return self._convertToOutputEncoding(xml, output_encoding=output_encoding)

InitializeClass(XMLRenderer)


def manage_addXMLRenderer(self, REQUEST=None):
    """ """
    try:
        self._setObject('xml_mods', XMLRenderer())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The renderer you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup
from Products.CMFCore.utils import getToolByName


class TestBibliographyFolder(PloneTestCase.PloneTestCase):
    '''Test the BibliographyFolder'''

    def afterSetUp(self):
        self._refreshSkinData()

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self, source_file=setup.MEDLINE_TEST_MED, format="med"):
        bf = self.getEmptyBibFolder()
        source = open(source_file, 'r').read()
        bf.processImport(source, 'source.%s' % format)
        return bf

    # the individual tests

    def test_FolderCreation(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'test_folder')
        self.failUnless('test_folder' in uf.contentIds())

    def test_MedlineImport(self):
        bf = self.getEmptyBibFolder()
        med_source = open(setup.MEDLINE_TEST_MED, 'r').read()
        bf.processImport(med_source, 'medline_test.med')
        expected_ids = ['GrootEtAl2003',
                        'AlibardiThompson2003',
                        'CokeEtAl2003',
                        'TrapeMane2002']
        for id in expected_ids:
            self.failUnless(id in bf.contentIds(),
                            'Importing %s failed.' % id)

    def test_BibtexImport(self):
        bf = self.getEmptyBibFolder()
        bib_source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        bf.processImport(bib_source, 'bibtex_test.bib')
        expected_ids = ['Lutz2001',
                        'Lattier2001',
                        'McKay2003']
        for id in expected_ids:
            self.failUnless(id in bf.contentIds(),
                            'Importing %s failed.' % id)
        # Test the annote handling
        self.failUnless(bf['McKay2003'].getAnnote() == 'I really like it.')

    def test_BibtexExport(self):
        bf = self.getPopulatedBibFolder()
        bib_source = bf.bibliography_export(format='BibTeX').strip()
        expected_source = open(setup.MEDLINE_TEST_BIB, 'r').read().strip()
        # just in case we need to debug that again        
        #l1 = bib_source.splitlines(1)
        #l2 = expected_source.splitlines(1)
        #from difflib import Differ
        #from pprint import pprint
        #d = Differ()
        #pprint(list(d.compare(l1,l2)))
        self.assertEqual(bib_source, expected_source)

    def test_BibtexExportCiteKeys(self):
        bf = self.getPopulatedBibFolder(setup.BIBTEX_TEST_CITE_KEY, 'bib')
        bib_source = bf.bibliography_export(format='BibTeX').strip()
        #expected_source = open(setup.BIBTEX_TEST_CITE_KEY, 'r').read().strip()
        self.failUnless(bib_source.startswith('@Book{Esping-Andersen1985'))

    def test_BibtexExportIgnoresNonBibrefItems(self):
        bf = self.getPopulatedBibFolder()
        bf.getPdfFolder() # non-bibref item
        bib_source = bf.bibliography_export(format='BibTeX').strip()
        expected_source = open(setup.MEDLINE_TEST_BIB, 'r').read().strip()
        self.assertEqual(bib_source, expected_source)

    # for the folder defaults
    def test_AuthorURLs(self):
        bf = self.getEmptyBibFolder()
        link_list = [{'key':'foo',
                      'value':'foos_home'},
                     {'key':'bar',
                      'value':'bars_home'}]
        bf.processAuthorUrlList(link_list)
        self.assertEqual(bf.AuthorURLs()['foo'], 'foos_home')
        self.assertEqual(bf.AuthorURLs()['bar'], 'bars_home')

    def test_getPublicationsByAuthors(self):
        bf = self.getPopulatedBibFolder()
        # one author
        refs  = bf.getPublicationsByAuthors('J Trape')
        self.assertEqual(len(refs), 1)
        self.assertEqual(refs[0].getId(), 'TrapeMane2002')
        # joint authors (and_flag false)
        refs  = bf.getPublicationsByAuthors(['J Trape',
                                             'Y Mane'])
        self.assertEqual(len(refs), 1)
        self.assertEqual(refs[0].getId(), 'TrapeMane2002')
        # joint authors (and_flag true)
        refs  = bf.getPublicationsByAuthors(['J Trape',
                                             'Y Mane'],
                                            1)
        self.assertEqual(len(refs), 1)
        self.assertEqual(refs[0].getId(), 'TrapeMane2002')
        # disjoint authors (and_flag false)
        refs  = bf.getPublicationsByAuthors(['J Trape',
                                             'T Groot'])
        ref_ids = [ref.getId() for ref in refs]
        self.assertEqual(len(refs), 2)
        self.failUnless('TrapeMane2002' in ref_ids)
        self.failUnless('GrootEtAl2003' in ref_ids)        
        # disjoint authors (and_flag true)
        refs  = bf.getPublicationsByAuthors(['J Trape',
                                             'T Groot'],
                                            1)
        self.assertEqual(len(refs), 0)

    def test_Top(self):
        bf = self.getPopulatedBibFolder()
        bf.setTop(['TrapeMane2002', 'GrootEtAl2003', 'CokeEtAl2003'])
        top_2 = [ref.getId() for ref in bf.Top(2)]
        self.assertEqual(top_2, ['TrapeMane2002', 'GrootEtAl2003'])

    def test_processImportReturnsObjects(self):
        bf = self.getEmptyBibFolder()
        med_source = open(setup.MEDLINE_TEST_MED, 'r').read()
        result = bf.processImport(med_source,
                                  'medline_test.med',
                                  return_obs=True,
                                  )
        self.assertEqual(len(result), 4)

    def test_cookId(self):
        bf = self.getEmptyBibFolder()
        ref_dict1 = {'authors' : [ {'lastname'  : 'Hicks',
                                    'firstname' : 'Tim',
                                    'middlename': 'M.',
                                    },
                                   {'lastname'  : 'Smith',
                                    'firstname' : 'Tom',
                                    'middlename': 'W.',
                                    },
                                 ],
                     'publication_year' : '2006'
                     }
        id1 = bf.cookId(ref_dict1)
        self.assertEqual(id1, 'HicksSmith2006')
        ref_dict2 = {'authors' : [ {'lastname'  : 'Hicks',
                                    'firstname' : 'Tim',
                                    'middlename': 'M.',
                                    },
                                   {'lastname'  : 'Smith',
                                    'firstname' : 'Tom',
                                    'middlename': 'W.',
                                    },
                                   {'lastname'  : 'Parsons',
                                    'firstname' : 'Tammy',
                                    'middlename': 'T.',
                                    },
                                 ],
                     'publication_year' : '2006'
                     }
        id2 = bf.cookId(ref_dict2)
        self.assertEqual(id2, 'HicksEtAl2006')

    def test_listDAVObjects(self):
        bf = self.getPopulatedBibFolder()
        # Get hold of a (random) reference object.
        an_obj = getattr(bf, 'GrootEtAl2003')
        # Check that listDAVObjects returns it.
        self.failUnless(an_obj in bf.listDAVObjects())
        # Transition the object to the 'private' state to limit access to it.
        wtool = getToolByName(bf, 'portal_workflow')
        wtool.doActionFor(an_obj, 'hide')
        # Give ourselves limited access
        self.logout()
        # Check that our newly-limited access stops us seeing the obj.
        self.failIf(an_obj in bf.listDAVObjects())

    # end of the individual tests


class TestLargeBibliographyFolder(TestBibliographyFolder):
    """Test the LargeBibliographyFolder.
    We subclass the TestBibliographyFolder test case so that all of those tests
    are applied to LargeBibliographyFolders as well.
    """

    def afterSetUp(self):
        self._refreshSkinData()
        # Make sure that 'LargeBibliographyFolder' is an addable type...
        ttool = getToolByName(self.folder, 'portal_types')
        ftype = getattr(ttool, 'Folder')
        ftype_allowed = list(ftype.getProperty('allowed_content_types'))
        ftype_allowed.append('LargeBibliographyFolder')
        ftype.manage_changeProperties(allowed_content_types=ftype_allowed)

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name='LargeBibliographyFolder',
                         id='bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self,
                              source_file=setup.MEDLINE_TEST_MED,
                              format='med'):
        bf = self.getEmptyBibFolder()
        source = open(source_file, 'r').read()
        bf.processImport(source, 'source.%s' % format)
        return bf

    # the individual tests


    # end individual tests


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBibliographyFolder))
    suite.addTest(makeSuite(TestLargeBibliographyFolder))
    return suite

if __name__ == '__main__':
    framework()



## Script (Python) "getAuthorDataFromMember"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=author
##title=
##

authorschema = author.Schema()
fname = ''
if authorschema.has_key('firstname'):
    fname = getattr(author, authorschema['firstname'].accessor)()
elif authorschema.has_key('given_name'):
    fname = getattr(author, authorschema['given_name'].accessor)()    
    
mname = ''
if authorschema.has_key('middlename'):
    mname = getattr(author, authorschema['middlename'].accessor)()
    
lname = ''
if authorschema.has_key('lastname'):
    lname = getattr(author, authorschema['lastname'].accessor)()
elif authorschema.has_key('surname'):
    lname = getattr(author, authorschema['surname'].accessor)()
    
hpage = ''
if authorschema.has_key('homepage'):
    hpage = getattr(author, authorschema['homepage'].accessor)()
elif authorschema.has_key('home_page'):
    hpage = getattr(author, authorschema['home_page'].accessor)()
else:
    hpage = '/'+context.portal_url.getRelativeContentURL(author)
    
if not fname and not lname:
    if authorschema.has_key('fullname'):
        fullname = getattr(author, authorschema['fullname'].accessor)().split(' ')
        fname = ' '.join(fullname[:-1])
        lname = ' '.join(fullname[-1:])

return lname and {'firstname':fname, 'middlename':mname, 'lastname':lname, 'homepage':hpage} or None



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Phdthesis reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, StringWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import schoolField, addressField, typeField


SourceSchema = Schema((
    typeField,
    schoolField,
    addressField,
     ))

PhdthesisSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
                  SourceSchema.copy() + TrailingSchema.copy()
PhdthesisSchema.get('authors').required = 1
PhdthesisSchema.get('school').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
PhdthesisSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class PhdthesisReference(BaseEntry):
    """ content type to make reference to a PhD thesis.
    """
    security = ClassSecurityInfo()
    archetype_name = "Phdthesis Reference"
    source_fields = ('school', 'address', 'publication_type',)

    schema = PhdthesisSchema

    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default source renderer
        """
        try:
            return self.PhdthesisSource()
        except AttributeError:
            publication_type  	= self.getPublication_type()
            school  		= self.getSchool()
            address 		= self.getAddress()
            
            if publication_type:
                source = publication_type
            else:
                source = 'PhD thesis'
	    if school: source += ', %s' % school
	    if address: source += ', %s' % address
	
	    return source + '.'

registerType(PhdthesisReference, PROJECTNAME)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
    Modul:          Entities
    Description:    Entitify and Deentitfy Specialchars for html/xml
    Version:        0.2
    Copyright:      2004 by Fritz Cizmarov fritz@sol.at
    Created:        29. Jul. 2004
    Last modified:  29. Jul. 2004
    License:        free
    Requirements:   Python2.3
    Exports:        Entities, entitify, deentitify
"""
"""
    2005.09.08: Adapted to CMFBib's needs (Raphael) 
"""

import re
import sys
from array import array
from htmlentitydefs import name2codepoint, codepoint2name

_c2n = {34 : "quot", 38 : "amp", 39 : "apos", 60 : "lt", 62 : "gt"}
_n2c = {"quot" : 34, "amp" : 38, "apos" : 39, "lt" : 60, "gt" : 62}

_regex = re.compile(r"&([#\w]+);", flags=re.UNICODE) # find entities


class Entities(object):
    """ Pseudomapping of entities """
   
    __slots__ = ["n2c","ukh"]
   
    def __init__(self, n2c={}, handle_unknown=lambda key: u"&%s;" % key ):
        super(Entities, self).__init__()
        self.n2c = n2c
        self.ukh = handle_unknown # standart restore entity
       
    def __getitem__(self, key):
        """ calculate unicode or get it from n2c """
        if key[0] == "#":   # numeric value
            if key[1] in "xX":  # hexadecimal
                return unichr(int(key[2:], 16))
            else:               # decimal
                return unichr(int(key[1:]))
        else: # named Entity
            try:
                return unichr(self.n2c[key])
            except KeyError:
                return self.ukh(key)


def entitify(ustring, c2n=_c2n):
    """ convert unicodestring to string with entities """
    tmp = array('c')
    if not isinstance(ustring, unicode):
        raise TypeError ("argument must be unicode")
    for char in ustring:
        cc = ord(char)
        if ( not (32 <= cc <= 127)) or c2n.has_key(cc):
            tmp.fromstring("&%s;" % c2n.get(cc, "#x%x" % cc))
        else: # simple ascii and not specialchar
            tmp.append(chr(cc))
    return tmp.tostring()


def deentitify(string, n2c=_n2c):
    """ convert string with entities to unicodestring """
    return _regex.sub(r"%(\1)s", string) % Entities(n2c)

class Convert:
    """Wrapper class for the entity module"""
    def deentitify(self, string, n2c=_n2c):
        return deentitify(string, n2c=_n2c)
    def entitify(self, ustring, c2n=_c2n):
        entitify(ustring, c2n=_c2n)
    def __call__(self, string=''):
        if _regex.search(string): # has entities?
            return deentitify(string, name2codepoint)
        else:
            return entitify(string.decode(sys.getfilesystemencoding()),
                            codepoint2name)

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print "usage: %s <string>"
        sys.exit()
    string = sys.argv[1]
    if _regex.search(string): # has entities?
        print deentitify(string, name2codepoint)
    else:
        print entitify(string.decode(sys.getfilesystemencoding()),
                       codepoint2name)




# Zope imports
from persistent.mapping import PersistentMapping

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from Products.ATExtensions.ateapi import FormattableNamesField

def replaceAuthorsField(obj):
    """
    change from the old custom 'publication_authors' field
    to the 'authors' FormattableNamesField from ATExtensions
    """
    try:
        obj.schema.replaceField('publication_authors',
                                FormattableNamesField('authors')
                                )
    except ValueError:
        pass

def migrateAuthorsValue(obj):
    """rename the attribute holding the value

    assumes attribute storage
    FormattableNamesField is clever enough to deal with the type change
    """
    authors = getattr(obj, 'publication_authors', None)
    if authors is not None:
        obj.authors = authors
        del obj.publication_authors
        obj._p_changed = 1
 
def migrateAuthors(obj):
    """
    deal with the renaming of the authors field:
    change the schema abd migrate the value
    """
    replaceAuthorsField(obj)
    migrateAuthorsValue(obj)

class Migration(object):
    """Migrating from 0.7 to 0.8

    It *must* be safe to use this multiple times as it is run automatically
    upon (re)install in the portal_quickinstaller.
    """

    def __init__(self, site, out):
        self.site = site
        self.out = out

    def migrate(self):
        """Run migration on site object passed to __init__.
        """
        print >> self.out
	print >> self.out, u"Migrating CMFBibliographyAT 0.7 -> 0.8"
	if self.needsAuthorSchemaUpgrade():
    	    self.migrateAuthorSchema()

    def needsAuthorSchemaUpgrade(self):
        """Returns True if one of the first 5 bibitems found
        has the old 'publication_authors' attribute; called
        by the installer to figure out whether a schema update
        is needed."""
	print >> self.out, u"authorSchema migration to FormattableNames field"
	print >> self.out, u"-----------------------------------------------"
	ct = getToolByName(self.site, 'portal_catalog')
        bib_tool = getToolByName(self.site, 'portal_bibliography')
	brains = ct(portal_type=bib_tool.getReferenceTypes(), Language='all')

	# needs schema upgrade for authors field
	if brains: 
	    for brain in brains[:5]:
		if getattr(brain.getObject(), 'publication_authors', False):
		    return True

	print >> self.out, u"    No authorSchema migration needed."
	print >> self.out
	return False
	    
    # migrate data from old to new schema
    def migrateAuthorSchema(self):
        """restore the old author data to be available to the
        new author schema"""
        ct = getToolByName(self.site, 'portal_catalog')
        bib_tool = getToolByName(self.site, 'portal_bibliography')
        brains = ct(portal_type=bib_tool.getReferenceTypes(), Language='all')
        for brain in brains:
            obj = brain.getObject()
            authors = getattr(obj, 'publication_authors', None)
            if authors is not None:
                print >> self.out, u"    migrating authors of objectId: %s" % obj.getId()
		migrateAuthors(obj)
	print >> self.out																								


## Python Script "getLastImportReport"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

report=[]
try:
    import_report = context.import_report
except AttributeError:
    # context must be an ATBiblioList
    import_report = context.getAssociatedBibFolder().import_report
old_report = import_report.split('\n')
for line in old_report:
    if line != '':
        if line != '='*30:
           report.append(line)
        else:
           return tuple(report)
return tuple(report)



## Script (Python) "get_bibliography_import_ftests"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Use case "Bibliography Import"

"""
Member adds bibfolder and imports a reference (two times)
(extended from 'usecase_login')
"""

selenium = context.portal_selenium
suite = selenium.getSuite()
target_language = 'en'
suite.setTargetLanguage(target_language)
selenium.addUser(id='sampleadmin',
                 fullname='Sample Admin',
                 roles=['Member', 'Manager',])
selenium.addUser(id='samplemember',
                 fullname='Sample Member',
                 roles=['Member',])

suite.addTests(
    "Bibliography import",
    "Login as member",
    suite.open("/logout"),
    suite.open('/'),
    suite.verifyElementPresent('portlet-login'),
    suite.verifyElementPresent('__ac_name'),
    suite.verifyElementPresent('__ac_password'),
    suite.type("__ac_name", 'samplemember'),
    suite.type("__ac_password", "admin"),
    suite.clickAndWait("submit"), # Should be name=login
    suite.verifyTextPresent("You are now logged in"),
    suite.verifyElementNotPresent("portlet-login"),
    "Add bibliography folder",
    suite.open("/Members/samplemember/createObject?type_name=BibliographyFolder&id=test_bib_folder"),
    suite.verifyTextPresent("Bibliography Folder has been created."),
    suite.type("title", "selenium test"),
    suite.clickAndWait("form_submit"),
    suite.verifyTextPresent("Changes saved."),
    "Import a reference",
    suite.open("/Members/samplemember/test_bib_folder/bibliography_importForm"),
    suite.type("up_text", """@Book{Lutz2001,
  author =       {Mark Lutz},
  authorURLs =   {http://home.rmi.net/~lutz/},
  title =        {Programming Python.},
  URL =          {http://home.rmi.net/~lutz/about-pp2e.html},
  abstract =     {This edition has been updated for Python version 2.
  More significantly, it is also an almost entirely new advanced Python topics
  book--a complete rewrite from the ground up. Very roughly, this edition includes:

    * 240 pages (4 chapters) on Systems Programming
    * 260 pages (4 chapters) on GUI Programming
    * 400 pages (6 chapters) on Internet Scripting
    * 150 pages (3 chapters) on Databases, Objects, and Text
    * 100 pages (2 chapters) on Python/C Integration

  To give you some idea of this edition's scope, it spans 1256 pages in its final
  published format, and includes some 446 program file examples, and 302 screen
shots
  (broken down here). See the table of contents link above for more content details.

  The book also includes a brand new CD-ROM with book examples, Python version 2.x
  release packages and installers, and related open source packages (NumPy, SWIG, PIL,
  PMW, and so on). Among other things, the book examples distribution on CD includes
  self-launching and Python-coded clocks, text editors, image viewers, email clients,
  calculators, and more.},
  publisher =    {O'Reilly},
  year =         2001,
  address =      {Sebastoplol, CA},
  edition =      {3rd}
  }"""),
    suite.clickAndWait("form.button.import"),
    suite.verifyTextPresent("Processed Entries: 1. Errors: 0."),
    "Import same reference again",
    suite.type("up_text", """@Book{Lutz2001,
  author =       {Mark Lutz},
  authorURLs =   {http://home.rmi.net/~lutz/},
  title =        {Programming Python.},
  URL =          {http://home.rmi.net/~lutz/about-pp2e.html},
  abstract =     {This edition has been updated for Python version 2.
  More significantly, it is also an almost entirely new advanced Python topics
  book--a complete rewrite from the ground up. Very roughly, this edition includes:

    * 240 pages (4 chapters) on Systems Programming
    * 260 pages (4 chapters) on GUI Programming
    * 400 pages (6 chapters) on Internet Scripting
    * 150 pages (3 chapters) on Databases, Objects, and Text
    * 100 pages (2 chapters) on Python/C Integration

  To give you some idea of this edition's scope, it spans 1256 pages in its final
  published format, and includes some 446 program file examples, and 302 screen
shots
  (broken down here). See the table of contents link above for more content details.

  The book also includes a brand new CD-ROM with book examples, Python version 2.x
  release packages and installers, and related open source packages (NumPy, SWIG, PIL,
  PMW, and so on). Among other things, the book examples distribution on CD includes
  self-launching and Python-coded clocks, text editors, image viewers, email clients,
  calculators, and more.},
  publisher =    {O'Reilly},
  year =         2001,
  address =      {Sebastoplol, CA},
  edition =      {3rd}
  }"""),
    suite.clickAndWait("form.button.import"),
    suite.verifyTextPresent("Processed Entries: 0. Errors: 1."),
    "Manage duplicates",
    suite.open("/Members/samplemember/test_bib_folder/bibliography_manage_duplicates"),
    suite.verifyTextPresent("Reference Duplication Management"),
    suite.test_delete_object(folder='/Members/samplemember',id='test_bib_folder'),
  )

return suite



from plone import PloneIdCooker, manage_addPloneIdCooker
from etal import EtalIdCooker, manage_addEtalIdCooker
from abbrev import AbbrevIdCooker, manage_addAbbrevIdCooker
from uid import UidIdCooker, manage_addUidIdCooker

def initialize(context):
    context.registerClass(PloneIdCooker,
                          constructors = (manage_addPloneIdCooker,),
                          ) 
    context.registerClass(EtalIdCooker,
                          constructors = (manage_addEtalIdCooker,),
                          ) 
    context.registerClass(AbbrevIdCooker,
                          constructors = (manage_addAbbrevIdCooker,),
                          )
    context.registerClass(UidIdCooker,
                          constructors = (manage_addUidIdCooker,),
                          )



## Script (Python) "isBibliographyExportable"
##title=Formats the history diff
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=obj=None

from Products.CMFCore.utils import getToolByName

#template = context.REQUEST.PUBLISHED
#template_id = template.getId()
iface = getToolByName(context, 'portal_interface')

#return template_id in ('bibliography_view', 'bibliography_search') or \
return iface.objectImplements(obj, 'Products.CMFBibliographyAT.interfaces.IBibliographyExport') 



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Proceedings reference class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, BooleanField
    from Products.LinguaPlone.public import StringWidget, BooleanWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, BooleanField
    from Products.Archetypes.public import StringWidget, BooleanWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import publisherField, addressField, volumeField, numberField, \
           organizationField, seriesField, editor_flagField
from Products.CMFBibliographyAT.content.book \
    import AuthorTrailingSchema

ThisAuthorTrailingSchema = AuthorTrailingSchema.copy()
ThisAuthorTrailingSchema['editor_flag'].default=1
ThisAuthorTrailingSchema['editor_flag'].widget.description='Leave this checked unless the people specified above are not the editors of this proceedings volume.'
ThisAuthorTrailingSchema['editor_flag'].widget.description_msgid='help_editor_flag_checked'
ThisAuthorTrailingSchema['editor_flag'].mode='r'

SourceSchema = Schema((
    publisherField,
    addressField,
    volumeField,
    numberField,
    organizationField,
    seriesField,
    ))

ProceedingsSchema = HeaderSchema.copy() + AuthorSchema.copy() + ThisAuthorTrailingSchema.copy() + \
                    CoreSchema.copy() + SourceSchema.copy() + TrailingSchema.copy()
ProceedingsSchema.get('authors').required = 0
ProceedingsSchema.get('publication_year').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
ProceedingsSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class ProceedingsReference(BaseEntry):
    """ content type to make reference to a book.
    """
    security = ClassSecurityInfo()
    archetype_name = "Proceedings Reference"
    source_fields = ('publisher', 'address', 'volume', 'number', 'organization', 'series',)

    schema = ProceedingsSchema
    
    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default proceedings source format
        """
        try:
	
            return self.ProceedingsSource()
        
	except AttributeError:
        
	    publisher 	 = self.getPublisher()
            address   	 = self.getAddress()
            volume    	 = self.getVolume()
            number       = self.getNumber()
            organization = self.getOrganization()
            series   	 = self.getSeries()
            
	    source = ''
            if publisher: source = ', %s' % publisher
            if address: source += ', ' + address
            if volume: 
		source += ', vol. %s' % volume
		if number: source += '(%s)' % number
	    if organization: source += ', %s' % organization

	    if source: source = source[2:]
	    if series: source += '. %s' % series
	    
            return source + '.'


registerType(ProceedingsReference, PROJECTNAME)



## Script (Python) "showISBNLink"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
site_default = context.portal_bibliography.getProperty('show_isbn_link', None)
return context.getProperty('show_isbn_link', site_default)



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Miscellanous reference class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, StringWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields import howpublishedField


howpublishedspecificField = howpublishedField.copy()
howpublishedspecificField.default = ''
SourceSchema = Schema((
    howpublishedspecificField,
     ))

MiscSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
             SourceSchema.copy() + TrailingSchema.copy()
MiscSchema.get('authors').required = 0
# normally the publication_year for MiscReferences is optional, but
# in CMFBAT we better force the user to enter something here (better not
# irritate portal_catalog...).
MiscSchema.get('publication_year').required = 1 

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
MiscSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class MiscReference(BaseEntry):
    """ content type to make reference to miscellaneous publications.
    """
    security = ClassSecurityInfo()
    archetype_name = "Misc Reference"
    source_fields = ('howpublished',)

    schema = MiscSchema
    
    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default misc source format
        """
        try:

            return self.MiscSource()

        except AttributeError:

            howpublished = self.getHowpublished()
	    
            source = 'Miscellaneous publication'
            if howpublished: source += ', %s' % howpublished
	    
            return source + '.'


registerType(MiscReference, PROJECTNAME)



##########################################################################
#                                                                        #
#           copyright (c) 2003, 2005 ITB, Humboldt-University Berlin     #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""BibliographyTool main class"""
import os.path

# Python stuff
import re, types, string


# Zope stuff
from Globals import InitializeClass, MessageDialog
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from OFS.Folder import Folder
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from persistent.mapping import PersistentMapping

try:
    import transaction
except ImportError:
    from Products.Archetypes import transaction

# CMF stuff
from Products.CMFCore.permissions import View, ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import UniqueObject

# My stuff ;-)
from Products.CMFBibliographyAT.tool.parsers.base import ParserFolder
from Products.CMFBibliographyAT.tool.renderers.base import RendererFolder
from Products.CMFBibliographyAT.tool.idcookers.base import IdCookerFolder
from Products.Archetypes.ArchetypeTool import ArchetypeTool
from Products.CMFBibliographyAT.DuplicatesCriteria import DuplicatesCriteriaManager
from Products.CMFBibliographyAT.config import REFERENCE_TYPES 
from Products.CMFBibliographyAT.config import FOLDER_TYPES as BIBFOLDER_TYPES
from Products.CMFBibliographyAT.config import ZOPE_TEXTINDEXES
from Products.CMFBibliographyAT import permissions

from Products.CMFBibliographyAT.utils import _encode, _decode

from bibutils import BibUtils

# citation patterns
citations = re.compile(r'\\?cite{([\w, ]*)}')
bibitems = re.compile(r'\\?bibitem{([\w]*)}')


class ImportParseError(Exception):
    """An exception to replace the use of TypeError in
    skins/bibliography/import.py as this masked true TypeErrors and made
    debugging that much more difficult.
    """
    pass

# Need to be able to import this exception so that it can be caught TTW.
module_path = 'Products.CMFBibliographyAT.tool.bibliography'
security = ModuleSecurityInfo(module_path)
security.declarePublic('ImportParseError')


class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
                       DuplicatesCriteriaManager, BibUtils):

    """Tool for managing import and export functionality
       as well as some resources of the BibliographyFolders
       and -Entries.
    """
    __allow_access_to_unprotected_subobjects__ = 1

    id = 'portal_bibliography'
    meta_type = 'Bibliography Tool'
    show_isbn_link = 0
    allow_folder_intro = 0
    bibfolders_translatable = 1
    bibrefitems_translatable = 1
    support_member_references = False
    default_idcooker_id = 'etal'
    use_pids_on_import = True
    cook_ids_on_bibref_creation = False
    cook_ids_after_bibref_edit = False
    synchronize_pdffile_attributes = False
    enable_duplicates_manager = True
    member_types = []
    sort_members_on = ''
    select_members_attr = ''
    members_search_on_attr = ''
    infer_author_references_after_edit = False
    infer_author_references_after_import = True
    authorof_implies_owner = False
    authorof_implies_creator = False
    allow_pdfupload_portal_policy = True
    allow_pdfupload_for_types = REFERENCE_TYPES
    searchable_bibfolders = True
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    manage_options = (
        (Folder.manage_options[0],)
        + DuplicatesCriteriaManager.manage_options
        + Folder.manage_options[2:]
        )

    _properties = Folder._properties + (
        {'id':'default_idcooker_id',
         'type':'selection',
	 'select_variable':'listIdCookers', 
         'mode':'w',
         },
        {'id':'cook_ids_on_bibref_creation',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'cook_ids_after_bibref_edit',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'use_pids_on_import',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'synchronize_pdffile_attributes',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'enable_duplicates_manager',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'allow_folder_intro',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'allow_pdfupload_portal_policy',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'allow_pdfupload_for_types',
         'type':'multiple selection',
         'select_variable':'getReferenceTypes',
         'mode':'w',
         },
        {'id':'bibfolders_translatable',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'bibrefitems_translatable',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'support_member_references',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'member_types',
         'type':'multiple selection',
         'select_variable':'getPortalTypeNames',
         'mode':'w',
         },
        {'id':'sort_members_on',
         'type':'selection',
         'select_variable':'getFieldIndexes',
         'mode':'w',
         },
        {'id':'select_members_attr',
         'type':'selection',
         'select_variable':'getMetaDataColumns',
         'mode':'w',
         },
        {'id':'members_search_on_attr',
         'type':'selection',
         'select_variable':'getTextIndexes',
         'mode':'w',
         },
        {'id':'infer_author_references_after_edit',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'infer_author_references_after_import',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'authorof_implies_owner',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'authorof_implies_creator',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'show_isbn_link',
         'type':'boolean',
         'mode':'w',
         },
        {'id':'preprint_servers',
         'type':'lines',
         'mode':'w',
         },
        {'id':'searchable_bibfolders',
         'type':'boolean',
         'mode':'w',
         },
        )
    _match_criteria = {}
    _default_criteria = ('bibliography type',)
    
    def __init__(self):
        self._setObject('Parsers', ParserFolder('Parsers', ''))
        self._setObject('Renderers', RendererFolder('Renderers', ''))
        self._setObject('IdCookers', IdCookerFolder('IdCookers', ''))
        DuplicatesCriteriaManager.__init__(self)
        # Add the local reference types registry
        self._reference_types = PersistentMapping()
        # Populate it initially with those types declared in config.py
        for ref_type in REFERENCE_TYPES:
            self._reference_types[ref_type] = None
     
    def isBibFolderTranslatable(self):
        """ return what is in bibfolders_translatable property
            this is only half of the truth!!! (ITranslatable)
        """    
        return self.bibfolders_translatable 

    def isBibrefItemTranslatable(self):
        """ return what is in bibrefitems_translatable property
            this is only half of the truth!!! (ITranslatable)
        """    
        return self.bibrefitems_translatable

    security.declarePublic('getMemberTypes')
    def getMemberTypes(self, default=()):
	member_types = self.getProperty('member_types', default)
	if type(member_types) not in (types.ListType, types.TupleType,):
	    return (member_types,)
	else:
	    return member_types
        
    security.declarePublic('getSortMembersOn')
    def getSortMembersOn(self, default='getId'):
	return self.getProperty('sort_members_on', default)
        
    security.declarePublic('getSelectMembersAttr')
    def getSelectMembersAttr(self, default='Title'):
	return self.getProperty('select_members_attr', default)
        
    security.declarePublic('getMembersSearchOnAttr')
    def getMembersSearchOnAttr(self, default='Title'):
	return self.getProperty('members_search_on_attr', default)
        
    security.declarePublic('getReferenceTypes')
    def getReferenceTypes(self, display=False):
        """
        returns a list with the names (meta types) of the
        currently registered reference types of a BibliographyFolder
        """
        if display:
            return DisplayList(tuple([ (key, key) for key in self.getReferenceTypes() ]))
        else:
            return self._reference_types.keys()

    security.declarePublic('getBibFolderTypes')
    def getBibFolderTypes(self):
        """
        returns a list with the names (meta types) of the
        currently registered bibfolder types
        """
        return BIBFOLDER_TYPES

    security.declareProtected(permissions.ManageReferenceTypes,
                              'registerReferenceType')
    def registerReferenceType(self, portal_type):
        """Add portal_type to the list that should be considered references
        for this plone instance.  Do not fail on duplicates.
        """
        # self._reference_types is a PersistentMapping
        self._reference_types[portal_type] = None

    security.declareProtected(permissions.ManageReferenceTypes,
                              'unregisterReferenceType')
    def unregisterReferenceType(self, portal_type):
        """Remove portal_type from the list that should be considered
        references for this plone instance.  Do not fail if the type is not
        already registered.
        """
        # self._reference_types is a PersistentMapping
        try:
            del self._reference_types[portal_type]
        except KeyError:
            pass 

    security.declarePublic('getImportFormatNames')
    def getImportFormatNames(self, with_unavailables=False, with_disabled=False):
        """
        returns a list with the names of the supported import formats
        """
        return [parser.getFormatName() \
		for parser in self.Parsers.objectValues() if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled) ]

    security.declarePublic('getImportFormatExtensions')
    def getImportFormatExtensions(self, with_unavailables=False, with_disabled=False):
        """
        returns a list with the file name extensions
        of the supported import formats
        """
        return [parser.getFormatExtension() \
                for parser in self.Parsers.objectValues() if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled)]

    def getImportFormatDescriptions(self, with_unavailables=False, with_disabled=False):
        """
        returns a list with the description texts
        of the supported import formats
        """
        return [parser.Description() \
                for parser in self.Parsers.objectValues() if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled) ]

    security.declarePublic('getExportFormatNames')
    def getExportFormatNames(self, with_unavailables=False, with_disabled=False):
        """
        returns a list with the names of the supported export formats
        """
        return [renderer.getFormatName() \
                for renderer in self.Renderers.objectValues() if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled) ]

    security.declarePublic('getExportFormatExtensions')
    def getExportFormatExtensions(self, with_unavailables=False, with_disabled=False):
        """
        returns a list with the file name extensions
        of the supported export formats
        """
        return [renderer.getFormatExtension() \
                for renderer in self.Renderers.objectValues() if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled) ]

    def getExportFormatDescriptions(self, with_unavailables=False, with_disabled=False):
        """
        returns a list with the description texts
        of the supported export formats
        """
        return [renderer.Description() \
                for renderer in self.Renderers.objectValues() if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled) ]

    security.declarePublic('getExportFormats')
    def getExportFormats(self, with_unavailables=False, with_disabled=False):
        """
        returns a list of (name, extension, description) tuples
        of the supported export formats
        """
        export_formats = zip(self.getExportFormatNames(with_unavailables=with_unavailables, with_disabled=with_disabled),
                             self.getExportFormatExtensions(with_unavailables=with_unavailables, with_disabled=with_disabled),
                             self.getExportFormatDescriptions(with_unavailables=with_unavailables, with_disabled=with_disabled))
        export_formats.sort()
        return export_formats                     
    
    security.declarePublic('getImportFormats')
    def getImportFormats(self, with_unavailables=False, with_disabled=False):
        """
        returns a list of (name, extension, description) tuples
        of the supported import formats
        """
        supported_parsers = zip(self.getImportFormatNames(with_unavailables=with_unavailables, with_disabled=with_disabled),
                                self.getImportFormatExtensions(with_unavailables=with_unavailables, with_disabled=with_disabled),
                                self.getImportFormatDescriptions(with_unavailables=with_unavailables, with_disabled=with_disabled))
        supported_parsers.sort()
        return supported_parsers
        
    security.declareProtected(View, 'render')
    def render(self, entry, format='', output_encoding=None, **kw):
        """
        renders a BibliographyEntry object in the specified format
        """
	renderer = self.getRenderer(format=format, **kw)
        if renderer:
            return renderer.render(entry, output_encoding=output_encoding, **kw)
        else:
            return None

    security.declareProtected(View, 'getRenderer')
    def getRenderer(self, format, with_unavailables=False, with_disabled=False, **kw):
        """
        returns the renderer for the specified format
        first looks for a renderer with the 'format' name
        next looks for a renderer with the 'format' extension
        """
        for renderer in self.Renderers.objectValues():
            if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled):
                if format.lower() == renderer.getFormatName().lower():
                    return renderer
                if format.lower() == renderer.getFormatExtension().lower():
                    return renderer
        return None

    security.declareProtected(View, 'getEntries')
    def getEntries(self, source, format, file_name=None):
        """
        main routine to be called from BibliographyFolders
        returns a list with the parsed entries
        """
        source = self.checkEncoding(source)

        format = self.checkFormat(source, format, file_name)

        parser = self.getParser(format)

        if parser:
            return parser.getEntries(source)
        else:
            return "No parser for '%s' available." % format

    security.declareProtected(View, 'getParser')
    def getParser(self, format, with_unavailables=False, with_disabled=False, **kw):
        """
        returns the parser for the specified format
        first looks for a parser with the 'format' name
        next looks for a parser with the 'format' extension
        """
        for parser in self.Parsers.objectValues():
            if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled):
                if format.lower() == parser.getFormatName().lower():
                    return parser
                elif format.lower() == parser.getFormatExtension().lower():
                    return parser
        return None ## rr: we probabliy should raise an error here 

    security.declareProtected(View, 'getIdCooker')
    def getIdCooker(self, idcooker_id='etal', with_disabled=False, **kwargs):
        """
        returns the id cooker object of the specified cooker_id
        """
        for idcooker in self.IdCookers.objectValues():
            if idcooker.isEnabled() or with_disabled:
                if idcooker_id == idcooker.getId():
                    return idcooker
        return None ## rr: we probabliy should raise an error here 

    security.declareProtected(View, 'getDefaultIdCooker')
    def getDefaultIdCooker(self, **kwargs):
        """
        returns the site wide default id cooker (as an object)
        """
	return self.getIdCooker(idcooker_id=self.default_idcooker_id, **kwargs)

    security.declareProtected(View, 'listIdCookers')
    def listIdCookers(self, with_disabled=False):
        """
        returns a list of strings with the ids of available id cookers
        """
        idcooker_ids = [ idcooker.getId() for idcooker in self.IdCookers.objectValues() if idcooker.isEnabled() or with_disabled ]
	idcooker_ids.sort()
	return idcooker_ids

    security.declareProtected(View, 'allowPdfUploadPortalPolicy')
    def allowPdfUploadPortalPolicy(self):
	
	return self.allow_pdfupload_portal_policy
        
    security.declareProtected(View, 'allowPdfUploadForTypes')
    def allowPdfUploadForTypes(self):
	
	return self.allow_pdfupload_for_types
        
    security.declareProtected(View, 'useParserIdsOnImport')
    def useParserIdsOnImport(self):
	
	return self.use_pids_on_import

    security.declareProtected(View, 'cookIdsOnBibRefCreation')
    def cookIdsOnBibRefCreation(self):
    
	return self.cook_ids_on_bibref_creation

    security.declareProtected(View, 'cookIdsAfterBibRefEdit')
    def cookIdsAfterBibRefEdit(self):
    
	return self.cook_ids_after_bibref_edit

    security.declareProtected(View, 'synchronizePdfFileAttributes')
    def synchronizePdfFileAttributes(self):
    
	return self.synchronize_pdffile_attributes

    security.declareProtected(View, 'searchableBibFolders')
    def searchableBibFolders(self):

        return self.searchable_bibfolders
    
    security.declareProtected(View, 'filterOutBibEntriesFromNonSearchableBibFolders')
    def filterOutBibEntriesFromNonSearchableBibFolders(self, search_results, object_getmethod='getObject'):

        result = []
        for b in search_results:
            object = getattr(b, object_getmethod)()
            if object.getParentNode().portal_type in self.getBibFolderTypes() and object.getSearchable():
                result.append(b)
        return result
    
    security.declareProtected(View, 'enableDuplicatesManager')
    def enableDuplicatesManager(self):
    
	return self.enable_duplicates_manager

    security.declareProtected(View, 'getEnableDuplicatesManager')
    def getEnableDuplicatesManager(self, instance=None):
    
        if instance and getattr(instance, 'portal_type', None) and instance.portal_type in ('BibliographyFolder', 'LargeBibliographyFolder',):
            return instance.getEnableDuplicatesManager()
        else:
    	    return self.enable_duplicates_manager

    security.declareProtected(View, 'inferAuthorReferencesAfterEdit')
    def inferAuthorReferencesAfterEdit(self):
    
	return self.infer_author_references_after_edit

    security.declareProtected(View, 'inferAuthorReferencesAfterImport')
    def inferAuthorReferencesAfterImport(self):
    
	return self.infer_author_references_after_import

    security.declareProtected(View, 'authorOfImpliesOwner')
    def authorOfImpliesOwner(self):
    
	return self.authorof_implies_owner

    security.declareProtected(View, 'authorOfImpliesCreator')
    def authorOfImpliesCreator(self):
    
	return self.authorof_implies_creator
    
    security.declareProtected(ManagePortal, 'transaction_savepoint')
    def transaction_savepoint(self, **kw):
	""" needed as a wrapper for bibliography_entry_cookId script
	"""
	transaction.savepoint(**kw)

    security.declareProtected(View, 'cookReferenceId')
    def cookReferenceId(self, ref, idcooker_id='etal', **kwargs):
        """
        returns the cooked id for reference object
        """
	idcooker = self.getIdCooker(idcooker_id=idcooker_id, **kwargs)

	if idcooker:
	    return idcooker.getCookedBibRefId(ref, **kwargs)
	    
	return 'nobody1000'

    def nextId(self, testid):
        letters = string.letters
	if testid[-1] in letters:
	    last = letters[letters.find(testid[-1])+1]
	    return testid[:-1] + last
	else:
	    # FIXME BAD
	    return testid + 'a'
	    
    def checkEncoding(self, source):
        """
        Make sure we have utf encoded text
        """
        mimetypesTool = getToolByName(self, 'mimetypes_registry', None)
        encoding = mimetypesTool and mimetypesTool.guess_encoding(source) \
                   or 'utf-8'
        fallbackEncoding = mimetypesTool and \
                           hasattr (mimetypesTool, 'fallbackEncoding') and \
                           mimetypesTool.fallbackEncoding or 'latin1'
        try:
            source = unicode(source, encoding)
        except UnicodeDecodeError:
            source = unicode(source, fallbackEncoding)
        return source.encode('utf-8')

    def checkFormat(self, source, format, file_name):
        """
        plausibility test whether 'source' has the 'format' specified
        if not it tries to infer the format from the 'file_name'
        raises an error if both fail
        """
        ok = 0
        if format:
            parser = self.getParser(format)
            ok = parser.checkFormat(source)
        if not ok and file_name:
            format = self.guessFormat(file_name)
        if format:
            return format
        else:
            raise  ImportParseError, "%s Parser's 'checkFormat' and " \
                  "guessing the format from the file name '%s' failed." \
                  % (format, file_name)

    def guessFormat(self, file_name):
        """
        Checks whether the file_name extension is
        among the supported ones.

        returns the respective format name if found 
        returns None otherwise
        """
        extension = file_name.split('.')[-1].lower()
        if extension in [ext.lower()
                         for ext in self.getImportFormatExtensions()]:
            for parser in self.Parsers.objectValues():
                if extension == parser.getFormatExtension().lower():
                    return parser.getFormatName()
        return None
    

    # support BibTeX style citations in text
    security.declarePublic('link_citations')
    def link_citations(self, text=""):
        """
        replace all citations with links to their references
        
        the pattern is 'cite{key}' or 'cite{key1,key2}
        If 'key' matches the id of a reference the pattern
        will be replaced with inline link(s) to this reference(s).
        Otherwise the pattern is replaced with the key.

        Using the pattern 'bibitem{key}' you can include the
        full reference like shown in other bibliographic listings
        (authors (year) title linked to entry, source) 
        """
        text = citations.sub(self._inline_links, text)
        return bibitems.sub(self._bibitem_links, text)

    def _inline_links(self, hit):
        keys = [k.strip() for k in hit.group(1).split(',')]
        results = []
        catalog = getToolByName(self, 'portal_catalog')
        encoding = self.getProperty('default_charset') or 'utf-8'
        for key in keys:
            brains = catalog(getId = key,
                             portal_type = self.getReferenceTypes()
                             )
            if brains:
                url = brains[0].getURL()
                label = _encode(brains[0].citationLabel, encoding) \
                        or 'no label'
                link = '<a href="%s">%s</a>' % (url, label)
                results.append(link)
            else:
                results.append(key)
        return '; '.join(results)

    def _bibitem_links(self, hit):
        key = hit.group(1).strip()
        catalog = getToolByName(self, 'portal_catalog')
        brains = catalog(getId = key,
                         portal_type = self.getReferenceTypes()
                         )
        if brains:
            brain = brains[0]
            encoding = self.getProperty('default_charset') or 'utf-8'

            authors = _encode(brain.Authors, encoding)
            year = _encode(brain.publication_year, encoding)
            title = _encode(brain.Title, encoding)
            source = _encode(brain.Source, encoding)
            url = brain.getURL()

            link = '%s (%s) <a href="%s">%s</a>, %s' % \
                   (authors, year, url, title, source)
            return link
        else:
            return key

    ## XXX just to have this in trusted code
    def getSortedMemberIds(self):
        """Return the ids like the membership tool but sorted (by id)"""
        mt = getToolByName(self, 'portal_membership')
        ids = mt.listMemberIds()
        ids.sort()
        return ids
    
    security.declareProtected(View, 'getFieldIndexes')
    def getFieldIndexes(self, start_with_empty_element=False):
        """returns a list of all field index ids from the catalog"""
        catalog = getToolByName(self, 'portal_catalog')
        indexes = catalog.indexes()
        field_indexes = [i for i in indexes \
                         if catalog.Indexes[i].meta_type == 'FieldIndex']
        field_indexes.sort()
        if start_with_empty_element:
            return [''] + field_indexes
        else:
            return field_indexes    

    security.declareProtected(View, 'getSortIndexes')
    def getSortIndexes(self, start_with_empty_element=False):
        """returns a list of all sortable index ids from the catalog"""
        catalog = getToolByName(self, 'portal_catalog')
        indexes = catalog.indexes()
        sort_indexes = [i for i in indexes \
                         if catalog.Indexes[i].meta_type in ('FieldIndex', 'DateIndex', 'DateRangeIndex', 'KeywordIndex') ]
        sort_indexes.sort()
        if start_with_empty_element:
            return [''] + sort_indexes
        else:
            return sort_indexes    


    security.declareProtected(View, 'filterOutDupesByTitle')
    def filterOutDupesByTitle(self, search_results):
        """ Filter out bibs with identical titles (dupe control) """
        result = []
        titles_seen = []
        for b in search_results:
            title = b.Title
            if not title in titles_seen:
                result.append(b)
                titles_seen.append(b.Title)
        return result


    security.declareProtected(View, 'getTextIndexes')
    def getTextIndexes(self, start_with_empty_element=False):
        """returns a list of all text index like ids from the catalog"""
        catalog = getToolByName(self, 'portal_catalog')
        indexes = catalog.indexes()
        text_indexes = [ i for i in indexes \
                          if catalog.Indexes[i].meta_type in ZOPE_TEXTINDEXES ]
        text_indexes.sort()
        if start_with_empty_element:
            return [''] + text_indexes
        else:
            return text_indexes    

    security.declareProtected(View, 'getMetaDataColumns')
    def getMetaDataColumns(self, start_with_empty_element=False):
        """returns a list of all field index ids from the catalog"""
        catalog = getToolByName(self, 'portal_catalog')
        columns = catalog.schema()
        if start_with_empty_element:
            return [''] + columns
        else:
            return columns   
            
    security.declareProtected(ManagePortal, 'getBibliographyContentTypes')
    def getBibliographyContentTypes(self):
        """ return a sequence that contains all bibliography types"""
        bibContentTypes = []
        arche_tool = getToolByName(self,'archetype_tool')
        bib_tool = getToolByName(self, 'portal_bibliography')
        for reference_type in bib_tool.getReferenceTypes():
            for reg_type in arche_tool.listRegisteredTypes():
                if reference_type == reg_type['name']:
                    bibContentTypes.append(reg_type)
        return bibContentTypes
													        
    security.declareProtected(ManagePortal, 'getBibliographyContentTypes')
    def getBibrefArchetypeName(self, classname):
        """ return the archetype name of a bibref content type class"""
        arche_tool = getToolByName(self,'archetype_tool')
        bib_tool = getToolByName(self, 'portal_bibliography')
        if classname in bib_tool.getReferenceTypes():
            for reg_type in arche_tool.listRegisteredTypes():
                if classname == reg_type['name']:
		    return reg_type['klass'].archetype_name
													        
    security.declareProtected(View, 'usesCMFMember')
    def usesCMFMember(self):
        quickinstaller = getToolByName(self, 'portal_quickinstaller')
        return quickinstaller.isProductInstalled('CMFMember')

    security.declarePrivate('getEntryDict')
    def getEntryDict(self, bibref_item, instance=None, title_link=False, title_link_only_if_owner=False, ):
        """ transform a BiblioRef Object into python dictionary
        """
        ref_attributes = ('publication_year',
                          'publication_month',
                          'publication_url',
                          'abstract',
                          'note',
		          'publisher',
			  'editor',
                          'editor_flag',
                          'organization',
                          'institution',
	                  'school',
	                  'address',
	                  'booktitle',
                          'chapter',
	                  'journal',
		          'volume',
                          'edition',
                          'number',
                          'pages',
	                  'series',
	                  'type',
	                  'howpublished',
	                  'preprint_server',
	                  'pmid',
                          'isbn',
			  'annote',)
	values = {}
	tmp_title = _decode(bibref_item.Title())
	if tmp_title and (tmp_title[-1] == '.'): tmp_title = tmp_title[:-1]
        values['title'] = tmp_title
        uniauthors=[]
        for author in bibref_item.getAuthorList():
    	    uniauthor={}
	    for field in author.keys():
	        uniauthor[field] = _decode(author.get(field))
	    uniauthors.append(uniauthor)
	values['authors'] = uniauthors
	uniauthors=[]
	uniauthor={}
	for attr in ref_attributes:
            field = bibref_item.getField(attr)
	    if field:
    		value = getattr(bibref_item, field.accessor)()
		if not value:
                    value = field.getDefault(bibref_item)
                try:
		    for x in range(value.len()):
		        value[x] = _decode(value[x])
	            values[attr] = value
	        except:
		    if type(value) == type(''):
            		values[attr] = _decode(value)
		    else:
		        # this is mainly for the editor flag (not a string, no unicode transformation possible
			values[attr] = value
        values['source'] = _decode(bibref_item.Source())
        values['description'] = _decode(bibref_item.Description())
        values['meta_type'] = bibref_item.meta_type
	values['reference_type'] = bibref_item.getReference_type() # should be same as portal_type
        if title_link:
            mtool = getToolByName(self, 'portal_membership')
            bibref_item_roles = mtool.getAuthenticatedMember().getRolesInContext(bibref_item)
            if not title_link_only_if_owner or [ role for role in ['Owner', 'Manager'] if role in bibref_item_roles ]:
	        values['title_link'] = bibref_item.absolute_url()
	wf_tool = getToolByName(self, 'portal_workflow')	
	values['review_state'] = wf_tool.getInfoFor(bibref_item, 'review_state', '')
	if instance is not None:
	    values['came_from'] = instance.absolute_url()
	    values['came_from_title'] = _decode(instance.Title())
	    values['came_from_description'] = _decode(instance.Description())
	return values

    def listWfStatesForReferenceTypes(self, filter_similar=1):

        wtool = getToolByName(self, 'portal_workflow')
        bibref_chain = []
        for reftype in self.getReferenceTypes():
            bibref_chain.extend(list(wtool.getChainForPortalType(reftype)))

        uniq_chain = []
        dummy = [ uniq_chain.append(ch) for ch in bibref_chain if ch not in uniq_chain ]

        bibref_states = []
        for statesByWorkflow in [ wtool.getWorkflowById(wf_id).states.objectValues() for wf_id in uniq_chain if wtool.getWorkflowById(wf_id) ]:
            bibref_states.extend(statesByWorkflow)

	# TODO: When using CMFPlacefulWorkflow the retrieved bibref_states might be an empty list.
	#       For this case let us for now just return the standard workflow states
	if not bibref_states:
	    return [ ('Private', 'private'), ('Public Draft', 'visible'), ('Pending', 'pending'), ('Published', 'published'), ]

        result = []
        dup_keys = {}
        for state in bibref_states:

            key = '%s:%s' % (state.title, state.getId(), )
            if not filter_similar:
                result.append((state.title, state.getId(), ))
            else:
                if not dup_keys.has_key(key):
                    result.append((state.title, state.getId(), ))
                dup_keys[key] = 1

        return result


InitializeClass(BibliographyTool)



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup

class TestContentTypes(PloneTestCase.PloneTestCase):
    '''Test the various content types'''

    def afterSetUp(self):
        self._refreshSkinData()

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self):
        bf = self.getEmptyBibFolder()
        med_source = open(setup.MEDLINE_TEST_MED, 'r').read()
        bf.processImport(med_source, 'medline_test.med')
        return bf

    # the individual tests

    def test_plainTextAbstract(self):
        bf = self.getPopulatedBibFolder()
        article = getattr(bf, 'GrootEtAl2003')
        abstract = "\nMy special abstract\n"
        article.setAbstract(abstract, mimetype='text/plain')
        self.assertEqual(str(article.editAbstract()), abstract)

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestContentTypes))
    return suite

if __name__ == '__main__':
    framework()



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Manual reference class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, StringWidget
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import organizationField, addressField, editionField


SourceSchema = Schema((
    organizationField,
    addressField,
    editionField,
     ))

ManualSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
               SourceSchema.copy() + TrailingSchema.copy()
ManualSchema.get('authors').required = 0
# normally the publication_year for ManualReferences is optional, but
# in CMFBAT we better force the user to enter something here (better not
# irritate portal_catalog...).
ManualSchema.get('publication_year').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
ManualSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class ManualReference(BaseEntry):
    """ content type to make reference to a manual.
    """
    archetype_name = "Manual Reference"
    source_fields = ('organization', 'address', 'edition',)
    security = ClassSecurityInfo()
    schema = ManualSchema
    
    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default manual source format
        """
        try:

            return self.ManualSource()

        except AttributeError:

	    bs_tool = getToolByName(self, 'portal_bibliostyles', None)
	    address = self.getAddress()
	    edition = self.getEdition()
	    
	    source = ''
	    if address: 
	        source += '%s' % address
		if edition: source += ', '
	    if edition: 
		if bs_tool: source += '%s' % bs_tool.formatEdition(edition, abbreviate=True)
		else: source += '%s ed.' % edition
	    if source and (source[-1] not in  '.?!'): source += '.'

	    return source


registerType(ManualReference, PROJECTNAME)



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Inproceedings reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
else:
    from Products.Archetypes.public import Schema
from Products.Archetypes.public import registerType
    
from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.inbook import InbookReference
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import booktitleField, editorField, volumeField, numberField, \
	   publisherField, organizationField, addressField, \
           editionField, seriesField, chapterField, pagesField

SourceSchema = Schema((
   booktitleField,
   editorField,
   volumeField,
   numberField,
   chapterField,
   pagesField,
   publisherField,
   organizationField,
   addressField,
   editionField,
   seriesField,
))
						       
InproceedingsSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
               SourceSchema.copy() + TrailingSchema.copy()
InproceedingsSchema.get('authors').required = 1
InproceedingsSchema.get('booktitle').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
InproceedingsSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }
								      
class InproceedingsReference(InbookReference):
    """ content type to make reference to a chapter within a proceedings volume.
    """
    security = ClassSecurityInfo()
    source_fields = ('booktitle', 'editor', 'volume', 'number',  'chapter', 'pages', 'publisher', 'organization', 'address', 'edition', 'series',)

    archetype_name = "Inproceedings Reference"
    schema = InproceedingsSchema

    security.declareProtected(View, 'Source')
    def Source(self):
	""" the default 'inproceedings' source format
        """
	try:

    	    return self.InproceedingsSource()

        except AttributeError:

	    booktitle        = self.getBooktitle()
	    editor           = self.getEditor()
	    volume           = self.getVolume()
    	    number           = self.getNumber()
	    chapter          = self.getChapter()
    	    pages            = self.getPages()
    	    publisher        = self.getPublisher()
	    organization     = self.getOrganization()
    	    address          = self.getAddress()
    	    edition          = self.getEdition()
    	    series           = self.getSeries()
    
	    bs_tool = getToolByName(self, 'portal_bibliostyles', None)
        
	    source = 'In: %s' % booktitle
	    if editor: source += ', ed. by %s' % editor
    	    if volume:
		source += ', vol. %s' % volume
        	if number: source += '(%s)' % number
	    if chapter: source += ', chap. %s' % chapter
	    if pages: source += ', pp. %s' % pages
    	    if organization: source += ', %s' % organization
    	    if address: source += ', %s' % address
    	    if publisher: source += ', %s' % publisher
            if edition:
                if bs_tool:
                    source += ', %s' % bs_tool.formatEdition(edition, abbreviate=True)
                else:
                    source += ',  %s ed.' % edition
	    
	    if source and (source[-1] not in '.!?'): source += '.'

	    if series: source += ' %s.' % series

	    return source

registerType(InproceedingsReference, PROJECTNAME)



##########################################################################
#                                                                        #
#           copyright (c) 2003, 2006 ITB, Humboldt-University Berlin     #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""package installer"""

import sys

from Globals import package_home

from Products.CMFCore import utils as CMFutils
from Products.CMFCore.DirectoryView import registerDirectory

from Products.Archetypes.public import process_types, listTypes

from config import SKINS_DIR, GLOBALS, PROJECTNAME
from config import ADD_CONTENT_PERMISSION, ENTRY_TYPES

import content
from tool import bibliography, parsers, renderers

tools = ( bibliography.BibliographyTool, )

registerDirectory(SKINS_DIR, GLOBALS)

def initialize(context):
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)
    CMFutils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)
    CMFutils.ToolInit(
        'Bibliography Tool',
        tools=tools,
        ## product_name=PROJECTNAME,
        icon='bibliography_tool.png',
        ).initialize(context)
    parsers.initialize(context)
    renderers.initialize(context)

# module aliases for backward compatibility

from content import base
sys.modules['Products.CMFBibliographyAT.BaseEntry'] = base
from content import folder
sys.modules['Products.CMFBibliographyAT.BibliographyFolder'] = folder

import content
sys.modules['Products.CMFBibliographyAT.bibtex_types'] = content
import DuplicatesCriteria
sys.modules['Products.CMFBibliographyAT.ImportCriterias'] = DuplicatesCriteria

old_path = 'Products.CMFBibliographyAT.bibtex_types.'
from content import article, booklet, book, inbook, incollection, \
     inproceedings, manual, mastersthesis, misc, phdthesis, preprint, \
     proceedings, techreport, unpublished, webpublished
sys.modules[old_path + 'ArticleReference'] = article
sys.modules[old_path + 'BookletReference'] = booklet
sys.modules[old_path + 'BookReference'] = book
sys.modules[old_path + 'InbookReference'] = inbook
sys.modules[old_path + 'IncollectionReference'] = incollection
sys.modules[old_path + 'InproceedingsReference'] = inproceedings
sys.modules[old_path + 'ManualReference'] = manual
sys.modules[old_path + 'MastersthesisReference'] = mastersthesis
sys.modules[old_path + 'MiscReference'] = misc
sys.modules[old_path + 'PhdthesisReference'] = phdthesis
sys.modules[old_path + 'PreprintReference'] = preprint
sys.modules[old_path + 'ProceedingsReference'] = proceedings
sys.modules[old_path + 'TechreportReference'] = techreport
sys.modules[old_path + 'UnpublishedReference'] = unpublished
sys.modules[old_path + 'WebpublishedReference'] = webpublished

from tool import bibliography
sys.modules['Products.CMFBibliographyAT.BibliographyTool'] = bibliography
from tool.parsers import base
sys.modules['Products.CMFBibliographyAT.BibliographyParser'] = base
from tool.renderers import base
sys.modules['Products.CMFBibliographyAT.BibliographyRenderer'] = base

from tool import parsers
sys.modules['Products.CMFBibliographyAT.parsers'] = parsers
old_path = 'Products.CMFBibliographyAT.parsers.'
from tool.parsers import bibtex, endnote, ibss, isbn, medline, pyblbibtex, \
     ris, xml
sys.modules[old_path + 'BibtexParser'] = bibtex
sys.modules[old_path + 'EndNoteParser'] = endnote
sys.modules[old_path + 'IBSSParser'] = ibss
sys.modules[old_path + 'ISBNParser'] = isbn
sys.modules[old_path + 'MedlineParser'] = medline
sys.modules[old_path + 'PyBlBibtexParser'] = pyblbibtex
sys.modules[old_path + 'RISParser'] = ris
sys.modules[old_path + 'XMLParser'] = xml

from tool import renderers
sys.modules['Products.CMFBibliographyAT.renderers'] = renderers
old_path = 'Products.CMFBibliographyAT.renderers.'
from tool.renderers import bibtex, endnote, pdf, ris, xml
sys.modules[old_path + 'BibtexRenderer'] = bibtex
sys.modules[old_path + 'EndRenderer'] = endnote
sys.modules[old_path + 'PDFRenderer'] = pdf
sys.modules[old_path + 'RISRenderer'] = ris
sys.modules[old_path + 'XMLRenderer'] = xml




############################################################################
#                                                                          #
#             copyright (c) 2004, 2006 ITB, Humboldt-University Berlin     #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""RISParser class (Research Information Systems/Reference Manager)"""

# Python stuff
import os

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser

#try:
#    import _bibtex
#    from Products.CMFBibliographyAT.tool.parsers.pyblbibtex \
#         import PyBlBibtexParser as BaseParser
from Products.CMFBibliographyAT.tool.parsers.bibtex \
     import BibtexParser as BaseParser


month_mapper = {'jan' : '01',
                'feb' : '02',
                'mar' : '03',
                'apr' : '04',
                'may' : '05',
                'jun' : '06',
                'jul' : '07',
                'aug' : '08',
                'sep' : '09',
                'oct' : '10',
                'nov' : '11',
                'dec' : '12',
               }

def fixMonth(s):
    s_lower = s.lower()
    if month_mapper.has_key(s_lower):
        return month_mapper[s_lower]
    return s


class RISParser(BaseParser):
    """
    A specific parser to process input in RIS format (Research Information Systems/Reference Manager).
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "RIS Parser"

    format = {'name':'RIS',
              'extension':'ris'}

    def __init__(self,
                 id = 'ris',
                 title = "RIS format parser"
                 ):
        """
        initializes including the regular expression patterns
        """
        BaseParser.__init__(self, id=id, title=title)   

    # Here we need to provide 'checkFormat' and 'preprocess'

    def isAvailable(self):
        """ test if transforming from RIS format to BibTex is possible...
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return bib_tool.isTransformable('ris', 'bib')
    
    def checkFormat(self, source):
        """
        is this RIS format?
        (Research Information Systems/Reference Manager)
        """
        teststring = source[:200].lower()
        ai = teststring.find('AU ')
        ei = teststring.find('TI ')
        di = teststring.find('PY ')
        if ai + ei + di > -2:
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        convert RIS to BibTeX
        """
        tool = getToolByName(self, 'portal_bibliography')
        return tool.transform(source, 'ris', 'bib')

    def parseEntry(self, entry):
        """See IBibliographyParser.
        """
        rd = super(RISParser, self).parseEntry(entry) # rd for 'result_dict'
        if not rd.has_key('number') and rd.has_key('issue'):
            rd['number'] = rd['issue'].replace(' ', '')
        rd['publication_month'] = fixMonth(rd['publication_month'])
        return rd
        
    # all the rest we inherit from our parent BibTeX(!) parser

InitializeClass(RISParser)

   
def manage_addRISParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('ris', RISParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')    
    return self.manage_main(self, REQUEST)



from setuptools import setup, find_packages

version = '0.9'

setup(name='Products.CMFBibliographyAT',
      version=version,
      description="Bibliographic references in Plone",
      long_description=open("Products/CMFBibliographyAT/README.txt").read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords=(
          'web application server zope zope2 cmf plone bibliography'),
      author='Raphael Ritz',
      author_email='raphael.ritz@incf.org',
      url='http://plone.org/products/cmfbibliographyat',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.DataGridField',
          'Products.ATExtensions',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      [zope2.initialize]
      Products.CMFBibliographyAT = Products.CMFBibliographyAT:initialize
      """,
      )



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys, types

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup, dummy
from Products.ATExtensions.ateapi import FormattableName, FormattableNames

class TestMemberAuthors(PloneTestCase.PloneTestCase):
    '''Test the reference types'''

    def afterSetUp(self):
        self._refreshSkinData()
        
	# set up the bibliography tool for member reference support
	uf = self.folder
	bib_tool = uf.portal_bibliography
	bib_tool.support_member_references = True
	bib_tool.member_types = ['SimpleTestMemberType', 'TestMemberType',]
	bib_tool.select_members_attr = 'Authors'
	bib_tool.members_search_on_attr = 'getAuthors'
	bib_tool.authorof_implies_owner = True
	bib_tool.authorof_implies_creator = True
	bib_tool.infer_author_references_after_edit = True
	bib_tool.infer_author_references_after_import = True

	# we need to fake the memberinfo from the member data tool
	uf.portal_membership.getMemberInfo = dummy.getMemberInfo

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self):
        bf = self.getEmptyBibFolder()
        med_source = open('medline_test.med', 'r').read()
        bf.processImport(med_source, 'medline_test.med')
        return bf   

    def setAuthorsFromMemberReferences(self, bibref_item=None, member_items=[]):
	edit_authors = []
	for member_item in member_items:    
	    edit_authors.append(FormattableName({'lastname': '', 'firstname': '', 'middlename': '', 'reference': member_item.UID(),},))
	edit_publication_year = '2000'
	request = {'authors': edit_authors, 'publication_year': edit_publication_year, }
	for key in request.keys():
	    self.app.REQUEST.set(key, request[key])
	bibref_item.validate(REQUEST=self.app.REQUEST)
	bibref_item.processForm(REQUEST=self.app.REQUEST)
	bibref_item.reindexObject()
	return bibref_item

    def createTestUserItems(self, here=None):
        
	if here:
	    here.invokeFactory('SimpleTestMemberType',
        	               'simpletestuser',
			       title='Firstname Lastname',
                               fullname='Firstname Lastname')
    	    simpletestuser = here.simpletestuser
    	    here.invokeFactory('TestMemberType',
                               'testuser',
			       title='First2 Middle2 Last2',
			       memberId='testlogin', # id and memberId differ intentionally!!!
                               firstname='First2',
                               middlename='Middle2',
                               lastname='Last2',
                               homepage='http://this.is')
            testuser = here.testuser
	    here.invokeFactory('SimpleTestMemberType',
        	               'mgabriel',
                               fullname='Mike Gabriel')
    	    here.invokeFactory('TestMemberType',
                               'psmusic',
			       title='Pia Siobhan Music',
			       memberId='psmusic',
                               firstname='Pia',
                               middlename='Siobhan',
                               lastname='Music',
                               homepage='http://zaubberer.net')
            testuser = here.testuser
	    return simpletestuser, testuser
	return None, None    

    # the individual tests

    def testEntryCreation(self):
    
        uf = self.folder

	simpletestuser, testuser = self.createTestUserItems(here=uf)

        bf = self.getEmptyBibFolder()
        bf.invokeFactory(type_name = 'ArticleReference',
                         id = 'test_article')		 
        self.failUnless('test_article' in bf.contentIds())
        article = bf.test_article
	
	article = self.setAuthorsFromMemberReferences(bibref_item=article, member_items=[simpletestuser])

        self.failUnlessEqual(article.Authors(format='%F %L'), 'Firstname Lastname')
        self.failUnlessEqual(article.Authors(format='%L, %F'), 'Lastname, Firstname')

	article = self.setAuthorsFromMemberReferences(bibref_item=article, member_items=[testuser])
        self.failUnlessEqual(article.Authors(format='%F %M %L'), 'First2 Middle2 Last2')
        self.failUnlessEqual(article.Authors(format='%L, %F %M'), 'Last2, First2 Middle2')

	article = self.setAuthorsFromMemberReferences(bibref_item=article, member_items=[testuser, simpletestuser,])
        self.failUnlessEqual(article.Authors(format='%F %M %L'), 'First2 Middle2 Last2 and Firstname Lastname')
        self.failUnlessEqual(article.Authors(format='%L, %F %M'), 'Last2, First2 Middle2 and Lastname, Firstname')

        self.failUnless(article.Authors())

    def testEntryLocalRoles(self):
    
        uf = self.folder
	simpletestuser, testuser = self.createTestUserItems(here=uf)
	
	bf = self.getEmptyBibFolder()
        bf.invokeFactory(type_name = 'ArticleReference',
                         id = 'test_article')		 
        self.failUnless('test_article' in bf.contentIds())
        article = bf.test_article
	
	article = self.setAuthorsFromMemberReferences(bibref_item=article, member_items=[simpletestuser, testuser,])
	article.inferAuthorReferences()

        self.failUnless('testlogin' in article.users_with_local_role('Owner'))
        self.failUnless('simpletestuser' in article.users_with_local_role('Owner'))

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMemberAuthors))
    return suite

if __name__ == '__main__':
    framework()



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Incollection reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.inbook import InbookReference
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import booktitleField, editorField, volumeField, numberField, \
	   typeField, publisherField, addressField, \
           editionField, seriesField, chapterField, pagesField
								      
SourceSchema = Schema((
    booktitleField,
    editorField,
    volumeField,
    numberField,
    typeField,
    publisherField,
    addressField,
    editionField,
    seriesField,
    chapterField,
    pagesField,
))
				    
IncollectionSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
        	     SourceSchema.copy() + TrailingSchema.copy()
IncollectionSchema.get('authors').required = 1
IncollectionSchema.get('publication_year').required = 1
IncollectionSchema.get('booktitle').required = 1
IncollectionSchema.get('publisher').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
IncollectionSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class IncollectionReference(InbookReference):
    """ content type to make reference to a chapter within a collection volume.
    """
    security = ClassSecurityInfo()
    source_fields = ('booktitle', 'editor', 'volume', 'number', 'publication_type', 'publisher', 'address', 'edition', 'series', 'chapter', 'pages',)

    archetype_name = "Incollection Reference"
    schema = IncollectionSchema

    security.declareProtected(View, 'Source')
    def Source(self):
	""" the default 'incollection' source format
	"""
	try:

    	    return self.IncollectionSource()

	except AttributeError:

            booktitle 	     = self.getBooktitle()
	    editor    	     = self.getEditor()
            volume    	     = self.getVolume()
            number    	     = self.getNumber()
            publication_type = self.getPublication_type()
            publisher 	     = self.getPublisher()
    	    address   	     = self.getAddress()
	    chapter   	     = self.getChapter()
	    pages     	     = self.getPages()
	    series     	     = self.getSeries()
    
    	    source = 'In: %s' % booktitle
            if editor: source += ', ed. by %s' % editor
	    if volume: 
    		source += ', vol. %s' % volume
                if number: source += '(%s)' % number
            if chapter: source += ', chap. %s' % chapter
    	    if pages: source += ', pp. %s' % pages
	    if publisher: source += ', ' + publisher
    	    if address: source += ', ' + address
	    if publication_type: source += ', %s' % publication_type   
    
	    if source and (source[-1] not in '.!?'): source += '.'
    
	    if series: source += ' %s.' % series
	    
	    return source

registerType(IncollectionReference, PROJECTNAME)
